#include "mgrbsa.h"

#include <math.h>    //fabs
// Useit et al...
#include <usebase/i18n.h>
#include <usebase/bsexception.h>
#include <usebase/bstrace.h>
#include <usebase/bsmath.h>
#include <usebase/bsdir.h>
#include <usebase/bsimagetools.h>

// BIZ
#include <biz/bzbobase.h>
#include <biz/bzbolist.h>
#include <biz/bzglobal.h>
#include <biz/bzexception.h>
#include <biz/bztsglobal.h>
#include <biz/bzbodeplist.h>
#include <zad/zdtransaction.h>
#include <zad/zdnorecord.h>

// Other subsystems
#include <ini/soini.h>
#include <inif/imgrinif.h>
#include <inif/inifid.h>
#include <gp/ibogeschpartnermmodul.h>
#include <gp/imgrbetrieb.h>

// Own includes
#include "boarbber.h"
#include "bolarbber.h"
#include "bointfzgart.h"
#include "botweintgf.h"
#include "bowaehrung.h"
#include "bozahlungsziel.h"
#include "bozahlungsmittel.h"
#include "bodruckerzuordnung.h"
#include "bosteuerbeh.h"
#include "bobetrieb2betrieb.h"
#include "bobetnrzuord.h"
#include "bomarkenvertrag.h"
#include "bodruckausgabekonfig.h"
#include "bofiskalbonnr.h"

#include "mgrfabrikat.h"

#include "bsaid_intern.h"

// my zad's
#include "zad_arbber.h"
#include "zad_abnehmer.h"
#include "zad_abnehmergrpbhcode.h"
#include "zad_steuerbehandlung.h"
#include "zad_umsatzsteuer.h"
#include "zad_standard.h"
#include "zad_stdwmand.h"
#include "zad_stdwarbb.h"
#include "zad_intfzgart.h"
#include "zad_tweintgf.h"
#include "zad_ngegf.h"
#include "zad_waehrung.h"
#include "zad_betriebdb.h"
#include "zad_zahlungsziel.h"
#include "zad_zhlgsmittel.h"
#include "zad_druckerzuordnung.h"
#include "zad_druckerzuordtagend.h"
#include "zad_druckerzuordstort.h"
#include "zad_drucker.h"
#include "zad_dokument.h"
#include "zad_tagendprog.h"
#include "zad_ngegf2ngegf.h"
#include "zad_mechgruppe.h"
#include "zad_betnrzuord.h"
#include "zad_kostenstelle.h"
#include "zad_laserdruckeinst.h"
#include "zad_druckerformular.h"
#include "zad_nwherstellergarantie.h"
#include "zad_stmgrenzen.h"
#include "zad_ngegfss.h"
#include "zad_lokationstandort.h"
#include "zad_ktsstandort.h"
#include "zad_extngegf.h"
#include "zad_programprotpos.h"
#include "zad_rechfixtext.h"
#include "zad_stbehandlust.h"
#include "zad_intfzgartherkgrpzuord.h"
#include "zad_verrechnungsart.h"
#include "zad_betrieb2betrieb.h"
#include "zad_markenvertrag.h"
#include "zad_druckausgabekonfig.h"
#include "zad_druckausgabekonfigexp.h"
#include "zad_fiskalbonnr.h"
#include "zad_behoerdennr.h"

#include <core/c2system.h>


#define DEFAULT_TOPMARGIN 17
#define DEFAULT_BOTTOMMARGIN 0

static InterfaceFactory<MgrBsa, IMgrBsa> oIMgrBsaFactory;
///////////////////////////////////////////////////////////////////////
BSPtr<IMgrBsa> IMgrBsa::Create()
{
   BSPtr<IMgrBsa> pIMgr(BZInterfaceFactory<IMgrBsa>::Create());
   return pIMgr;
}
////////////////////////////////////////////////////////////////////////////////
MgrBsa::MgrBsa()
       :IMgrBsa()
{
}

////////////////////////////////////////////////////////////////////////////////
MgrBsa::~MgrBsa()
{
}

void MgrBsa::MapSteuerlBeh(const BZBoBase& bo, AnsSteuerlBeh& ans) const
{
   ans.lfdnr               = bo.Get(STEUERBEHANDLUNG::LFDNR            );
   ans.bezeichnung         = bo.Get(STEUERBEHANDLUNG::BEZEICHNUNG      );
   ans.intern              = bo.Get(STEUERBEHANDLUNG::INTERN           );
   ans.einkverk            = bo.Get(STEUERBEHANDLUNG::EINKVERKKZ       );
   ans.gueltpers           = bo.Get(STEUERBEHANDLUNG::GUELTIGFUERPERS  ).Get(false);
   ans.ustiderforderl      = bo.Get(STEUERBEHANDLUNG::USTIDERFORDERL   );
   ans.innergemlieferung   = bo.Get(STEUERBEHANDLUNG::INNERGEMLIEFERUNG).Get(false);
   ans.lfdnreinkauf        = bo.Get(STEUERBEHANDLUNG::LFDNREINKAUF     );
   ans.ustueberstkz        = bo.Get(STEUERBEHANDLUNG::USTUEBERSTKZ     ).Get(false);
   ans.ustueberstcode      = bo.Get(STEUERBEHANDLUNG::USTUEBERSTCODE   );
   ans.auslandslieferung   = bo.Get(STEUERBEHANDLUNG::AUSLANDSLIEFERUNG).Get(false);
   ans.gueltigfuerkd       = bo.Get(STEUERBEHANDLUNG::GUELTIGFUERKD    ).Get(false);
   ans.gueltigfuernw       = bo.Get(STEUERBEHANDLUNG::GUELTIGFUERNW    ).Get(false);
   ans.gueltigfuergw       = bo.Get(STEUERBEHANDLUNG::GUELTIGFUERGW    ).Get(false);
   ans.betragsgrenze       = bo.Get(STEUERBEHANDLUNG::BETRAGSGRENZE    );
   ans.drucktext           = bo.Get(STEUERBEHANDLUNG::DRUCKTEXT        );

   ans.intrastatrelevant = (ans.innergemlieferung && !ans.gueltpers);
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetSteuerlBeh(long lfdnr, AnsSteuerlBeh& ans, bool incl_deleted)
{
   BSPtr<BZBoBase> pB(new BoSteuerBehandlung());

   pB->SetLong(STEUERBEHANDLUNG::LFDNR, lfdnr);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Steuerliche Behandlung mit DB-Key %1 nicht gefunden."),
                              BsaId::STBEH_NOTFOUND).Arg(lfdnr);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(STEUERBEHANDLUNG::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Steuerliche Behandlung mit DB-Key %1 ist als gelscht gekennzeichnet."),
                           BsaId::STBEH_NOTFOUND).Arg(lfdnr);
   }

   MapSteuerlBeh(*(pB.get()), ans);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetSteuerlBehListe(BSMap<long, IMgrBsa::AnsSteuerlBeh>& oANSMap, bool incl_deleted)
{
   BSPtr<BZBoBase>     pB(new BoSteuerBehandlung());
   BSPtr<ZDListAccess> pList(new STEUERBEHANDLUNG::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();
   for(int i=0; i<bolist.Count(); i++)
   {
      if(!incl_deleted && bolist.GetBO(i)->GetBoolNull(STEUERBEHANDLUNG::LOESCHKZ).Get())
         continue;

      IMgrBsa::AnsSteuerlBeh* pAns = new IMgrBsa::AnsSteuerlBeh();
      MapSteuerlBeh(*(bolist.GetBO(i)), *pAns);
      oANSMap.Insert(pAns->lfdnr.Get(), pAns);
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetSteuerlBeh(const REQUst&  req,
                           AnsSteuerlBeh& ans,
                           bool&          found)
{
   BSASSERT(!req.ustcode.IsEmpty());
   BSASSERT(!req.vondatum.IsNull());

   BSPtr<BZBoBase> pB(new BoSteuerBehandlung());
   BSPtr<ZDListAccess> pList(new STEUERBEHANDLUNG::ListeForMgr);
   BZBoList boList(pB, pList);

   boList.Param()->SetString(STEUERBEHANDLUNG::ListeForMgr::Param::USTCODE_EQ,    req.ustcode);
   boList.Param()->SetDate  (STEUERBEHANDLUNG::ListeForMgr::Param::DATUM_EQ,      req.vondatum.Get());
   boList.Param()->SetString(STEUERBEHANDLUNG::ListeForMgr::Param::EINKVERKKZ_EQ, req.einkverkkz);
   boList.Read();

   if(boList.Count() == 1)
   {
      MapSteuerlBeh(*(boList.GetBO(0)), ans);
      found = true;
      return;
   }
   else
      found = false;
}
////////////////////////////////////////////////////////////////////////////////
IMgrBsa::AnsSteuerlBeh MgrBsa::GetSteuerlBehGIFakt(bool incl_deleted)
{
   BSPtr<BZBoBase> pB(new BoSteuerBehandlung());
   BSPtr<ZDListAccess> pList(new STEUERBEHANDLUNG::Liste);
   BZBoList boList(pB, pList);
   boList.Param()->SetBool(STEUERBEHANDLUNG::Liste::Param::ZUORDNUNGGI_EQ, true);
   boList.Read();

   bool found = false;
   int  i;
   for(i=0; i<boList.Count(); i++)
   {
      if(!incl_deleted && boList.GetBO(i)->GetBoolNull(STEUERBEHANDLUNG::LOESCHKZ).Get() == true) 
         continue;

      found = true;
      break;
   }
   
   // Satz nicht gefunden
   if (!found)
   {
      BSTHROW BZException(i18n("Es wurde keine steuerliche Behandlung gefunden, bei der das Kennzeichen \"GI-Faktura\" gesetzt ist"),
                           BsaId::STBEH_NOTFOUND);
   }

   IMgrBsa::AnsSteuerlBeh oANS;
   MapSteuerlBeh(*(boList.GetBO(i)), oANS);

   return oANS;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetArbeitsbereich(const MModul& req, AnsArbBereich& ans, bool incl_ungueltig)
{
   BSPtr<BZBoBase> pB(new BoArbBereich);
   pB->SetInt(ARBBEREICH::MANDANTNR, req.mandantnr.Get());
   pB->SetInt(ARBBEREICH::MODULNR  , req.modulnr.Get());

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Der Arbeitsbereich (betmandmodul) mit Mandant %1, Modul %2 ist nicht definiert."),
            BsaId::ARBBEREICH_NOTFOUND)
            .Arg(req.mandantnr.Get(-1))
            .Arg(req.modulnr.Get(-1));
      }
      else
         throw;
   }

   BSDate today = BSDate::CurrentDate();
   if (pB->GetDateNull(ARBBEREICH::GUELTIGVON).Get() > today ||
       pB->GetDateNull(ARBBEREICH::GUELTIGBIS).Get() < today)
   {
      ans.isGueltig = false;
      if (!incl_ungueltig)
      {
         BSTHROW BZException(i18n("Fr Arbeitsbereich (betmandmodul) %1 (%2/%3) ist das Modul %4 nicht gltig."),
            BsaId::ARBBEREICH_NOTFOUND)
            .Arg(pB->Get(ARBBEREICH::ARBEITSBEREICH))
            .Arg(req.mandantnr.Get(-1))
            .Arg(req.modulnr.Get(-1))
            .Arg(csIni->GetWinText(csIni->GetNeutralKey(pB->Get(ARBBEREICH::TEXTKZ), L"MODUL"), L"MODUL"));
      }
   }
   else
   {
      ans.isGueltig = true;
   }

   MapArbBereich(*pB, ans);
}
void MgrBsa::GetArbeitsbereich(const ReqGptnr& req, AnsArbBereich& ans, bool incl_ungueltig)
{
   BSASSERT(!req.modulkz.IsEmpty());

   BSPtr<BZBoBase>     pB   (new BoArbBereich);
   BSPtr<ZDListAccess> pList(new ARBBEREICH::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();

   int index = bolist.GetIndex(ARBBEREICH::GPTNR , req.gptnr.Get(-1),
                               ARBBEREICH::TEXTKZ, req.modulkz);
   // Satz nicht gefunden
   if (index < 0)
      BSTHROW BZException(i18n("Fr Arbeitsbereich (betmandmodul) mit Gptnr %1 ist kein Modul %2 (Modulkz %3) definiert."),
         BsaId::ARBBEREICH_NOTFOUND)
         .Arg(req.gptnr.Get(-1))
         .Arg(csIni->GetWinText(csIni->GetNeutralKey(req.modulkz, L"MODUL"), L"MODUL"))
         .Arg(req.modulkz);

   const BZBoBase* pBo = bolist.GetBO(index);

   BSDate today = BSDate::CurrentDate();
   if (!incl_ungueltig &&
       (pBo->GetDateNull(ARBBEREICH::GUELTIGVON).Get() > today ||
        pBo->GetDateNull(ARBBEREICH::GUELTIGBIS).Get() < today))
   {
      BSTHROW BZException(i18n("Fr Arbeitsbereich (betmandmodul) %1 (Gptnr %2) ist das Modul %3(Modulkz %4) nicht gltig."),
         BsaId::ARBBEREICH_NOTFOUND)
         .Arg(bolist.GetBO(bolist.GetIndex(ARBBEREICH::GPTNR , req.gptnr.Get()))->Get(ARBBEREICH::ARBEITSBEREICH))
         .Arg(req.gptnr.Get())
         .Arg(csIni->GetWinText(csIni->GetNeutralKey(req.modulkz, L"MODUL"), L"MODUL"))
         .Arg(req.modulkz);
   }

   MapArbBereich(*pBo, ans);
}

void MgrBsa::GetArbeitsbereichListe(BSTCont<AnsArbBereich>& ans      ,
                                    const int_null&         mandantnr,
                                    const CString&          modulkz  ,
                                    const CString&          betnr    ,
                                    const BSDate&           gueltigDatum,
                                    const bool&             incl_ungueltig)
{
   ans.Clear();

   BSPtr<BZBoBase>     pB   (new BoArbBereich);
   BSPtr<ZDListAccess> pList(new ARBBEREICH::Liste);
   BZBoList bolist(pB, pList);

   if (incl_ungueltig)
   {
      bolist.Param()->SetNull(ARBBEREICH::Liste::Param::V_GUELTIG);
   }
   else
   {
      // Nur gltige Arbeitsbereiche:
      bolist.Param()->SetDate    (ARBBEREICH::Liste::Param::V_GUELTIG  , gueltigDatum);
   }
   bolist.Param()->SetIntNull (ARBBEREICH::Liste::Param::V_MANDANTNR, mandantnr);
   bolist.Param()->SetString  (ARBBEREICH::Liste::Param::V_MODULKZ  , modulkz  );
   bolist.Param()->SetString  (ARBBEREICH::Liste::Param::V_BETNR    , betnr    );
   bolist.Read();

   AnsArbBereich* ansarb;
   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {

      pBo = bolist.GetBO(i);
      ansarb = new AnsArbBereich;
      ans.Insert(ansarb);
      MapArbBereich(*pBo, *ansarb);
   }
}

void MgrBsa::GetArbeitsbereichListe(BSTCont<AnsArbBereich>& ans,
                                    const CString&          mmodul_str_sql,
                                    const BSDate&           gueltigDatum)
{
   ans.Clear();

   BSPtr<BZBoBase>     pB   (new BoArbBereich);
   BSPtr<ZDListAccess> pList(new ARBBEREICH::Liste);
   BZBoList bolist(pB, pList);
   // Nur gltige Arbeitsbereiche (fr Liste von Mandant-/Modul-Kombinationen) - fr Arbeitsbereich-Multiselect:
   bolist.Param()->SetDate    (ARBBEREICH::Liste::Param::V_GUELTIG,      gueltigDatum);
   bolist.Param()->SetString  (ARBBEREICH::Liste::Param::MMODUL_STR_SQL, mmodul_str_sql);
   bolist.Read();

   AnsArbBereich* ansarb;
   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);
      ansarb = new AnsArbBereich;
      ans.Insert(ansarb);
      MapArbBereich(*pBo, *ansarb);
   }
}

void MgrBsa::GetArbbeListeMitTextKz(BSTCont<AnsArbBereich>& ans,
                                    const CString&          textKz,
                                    const bool&     incl_ungueltig)
{
   ans.Clear();

   BSPtr<BZBoBase>     pB   (new BoArbBereich);
   BSPtr<ZDListAccess> pList(new ARBBEREICH::Auswahlliste);
   BZBoList bolist(pB, pList);

   bolist.Param()->SetBool    (ARBBEREICH::Auswahlliste::Param::NUR_GUELTIGE, !incl_ungueltig);
   bolist.Param()->SetString  (ARBBEREICH::Auswahlliste::Param::V_TEXTKZ,     textKz         );
   bolist.Read();

   AnsArbBereich* ansarb;
   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);
      ansarb = new AnsArbBereich;
      ans.Insert(ansarb);
      MapArbBereich(*pBo, *ansarb);
   }
}

void MgrBsa::GetLokationStandort( const MModul& req,
                                  const long lokationslfdnr,
                                  AnsLokationStandort& ans  )
{
   BSPtr<ZDDataStruct> pZD(new LOKATIONSTANDORT);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetIntNull(LOKATIONSTANDORT::MODULNR        , req.modulnr    );
   pB->SetIntNull(LOKATIONSTANDORT::MANDANTNR      , req.mandantnr  );
   pB->SetLong   (LOKATIONSTANDORT::LOKATIONLFDNR  , lokationslfdnr );

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Lokationstandort zu Lokationslfdnr %1 nicht gefunden."),
            BsaId::LOKATIONSTANDORTLFDNR_NOTFOUND).Arg(lokationslfdnr);
      }
      else
         throw;
   }

   ans.mandantnr     = pB->GetIntNull (LOKATIONSTANDORT::MANDANTNR     ).Get();
   ans.modulnr       = pB->GetIntNull (LOKATIONSTANDORT::MODULNR       ).Get();
   ans.lokationlfdnr = pB->GetLongNull(LOKATIONSTANDORT::LOKATIONLFDNR ).Get();
   ans.standortkz    = pB->GetString  (LOKATIONSTANDORT::STANDORT      );
}

void MgrBsa::GetLokationStandort( const MModul& req,
                                  const CString& arbeitsplatz,
                                  AnsLokationStandort& ans     )
{
   // wirft BZException InifId::ARBEITSPLATZ_NOTFOUND, falls arbeitsplatz nicht existiert ( nicht definiert ist )
   BSASSERT(!arbeitsplatz.IsEmpty());
   long lokationlfdnr = -1;

   IMgrInif::ANSArbeitsplatz oANSAplatz;
   IMgrInif::Create()->GetArbeitsplatz(arbeitsplatz, oANSAplatz);

   lokationlfdnr = oANSAplatz.lokationlfdnr.Get(-1);
   GetLokationStandort(req, lokationlfdnr, ans );
}

void MgrBsa::MapArbBereich(const BZBoBase& bo, AnsArbBereich& ans) const
{
   ans.mandantnr        = bo.GetIntNull (ARBBEREICH::MANDANTNR).Get();
   ans.modulnr          = bo.GetIntNull (ARBBEREICH::MODULNR  ).Get();
   ans.gptnr            = bo.GetLongNull(ARBBEREICH::GPTNR    ).Get();
   ans.betnr            = bo.GetString  (ARBBEREICH::BETNR    );
   ans.modulkz          = bo.GetString  (ARBBEREICH::TEXTKZ   );
   ans.arbeitsbereich   = bo.GetString  (ARBBEREICH::ARBEITSBEREICH);
   
   if(ans.modulkz == csIni->GetValue(L"NW", L"MODUL"))
      ans.arbBereich = AnsArbBereich::NW;
   if(ans.modulkz == csIni->GetValue(L"GW", L"MODUL"))
      ans.arbBereich = AnsArbBereich::GW;
   if(ans.modulkz == csIni->GetValue(L"KD", L"MODUL"))
      ans.arbBereich = AnsArbBereich::KD;
}
// ----------------------------------------------------------------------------
bool IMgrBsa::AnsArbBereich::IsNW() const
{
   return (csIni->GetNeutralKey(modulkz, L"MODUL") == L"NW");
}
bool IMgrBsa::AnsArbBereich::IsGW() const
{
   return (csIni->GetNeutralKey(modulkz, L"MODUL") == L"GW");
}
bool IMgrBsa::AnsArbBereich::IsKD() const
{
   return (csIni->GetNeutralKey(modulkz, L"MODUL") == L"KD");
}
// ----------------------------------------------------------------------------
bool MgrBsa::IsArbeitsbereichGueltig(const MModul& req)
{
   AnsArbBereich ansarb;
   GetArbeitsbereich(req, ansarb, true);

   return ansarb.isGueltig;
}
// ----------------------------------------------------------------------------
bool MgrBsa::CheckArbeitsbereich(const ReqGptnr& req)
{
   BSPtr<BZBoBase>     pB   (new BoArbBereich);
   BSPtr<ZDListAccess> pList(new ARBBEREICH::CheckArb);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetLong  (ARBBEREICH::CheckArb::Param::V_GPTNR  , req.gptnr.Get()  );
   bolist.Param()->SetString(ARBBEREICH::CheckArb::Param::V_MODULKZ, req.modulkz);
   bolist.Read();

   if (bolist.Count() <= 0)
      return false;
   return true;
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetAbnehmerGrp(const ReqAbnehmergrp& req, AnsAbnehmergrp& ans, bool incl_deleted)
{
   // Ermitteln ModulKz fr Mandant/Modul-Kombination
   AnsArbBereich ansarb;
   GetArbeitsbereich(req,ansarb);

   BSPtr<ZDDataStruct> pZD(new ABNEHMERGRP);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetString(ABNEHMERGRP::MODULKZ, ansarb.modulkz);
   pB->SetString(ABNEHMERGRP::ABNEHMERGRUPPE, req.abnehmergrp);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Abnehmergruppe %1 nicht gefunden."),
                              BsaId::ABNEHMGRP_NOTFOUND).Arg(req.abnehmergrp);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(ABNEHMERGRP::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Abnehmergruppe '%1' ist als gelscht gekennzeichnet."),
                           BsaId::ABNEHMGRP_NOTFOUND).Arg(req.abnehmergrp);
   }

   ans.bezeichnung =       pB->GetString(ABNEHMERGRP::BEZEICHNUNG);
   ans.nora =              pB->GetBoolNull(ABNEHMERGRP::NORA).Get();
   ans.mikabnehmergrp =    pB->GetString(ABNEHMERGRP::MIKABNEHMERGRUPPE);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetAbnehmerGrpListe(const CString& modulKZ, BSMap<CString, IMgrBsa::AnsAbnehmergrp>& oANSMap, bool incl_deleted)
{
   BSPtr<ZDDataStruct> pZD(new ABNEHMERGRP);
   BSPtr<BZBoBase>     pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ABNEHMERGRP::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(ABNEHMERGRP::Liste::Param::MODUL, modulKZ);
   bolist.Read();
   for(int i=0; i<bolist.Count(); i++)
   {
      if(!incl_deleted && bolist.GetBO(i)->GetBoolNull(ABNEHMERGRP::LOESCHKZ).Get())
         continue;

      IMgrBsa::AnsAbnehmergrp* pAns = new IMgrBsa::AnsAbnehmergrp();
      pAns->bezeichnung =       bolist.GetBO(i)->GetString(ABNEHMERGRP::BEZEICHNUNG);
      pAns->nora =              bolist.GetBO(i)->GetBoolNull(ABNEHMERGRP::NORA).Get();
      pAns->mikabnehmergrp =    bolist.GetBO(i)->GetString(ABNEHMERGRP::MIKABNEHMERGRUPPE);
      oANSMap.Insert(bolist.GetBO(i)->GetString(ABNEHMERGRP::ABNEHMERGRUPPE), pAns);
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetAbnehmerGrpMitModulKz(const ReqAbnehmergrp& req,
                                      const CString& modulKz,
                                      AnsAbnehmergrp& ans,
                                      bool incl_deleted)
{
   BSPtr<ZDDataStruct> pZD(new ABNEHMERGRP);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetString(ABNEHMERGRP::MODULKZ, modulKz);
   pB->SetString(ABNEHMERGRP::ABNEHMERGRUPPE, req.abnehmergrp);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Abnehmergruppe %1 nicht gefunden."),
                              BsaId::ABNEHMGRP_NOTFOUND).Arg(req.abnehmergrp);
      }
      else
         throw;
   }
   if (incl_deleted == false
       && pB->GetBoolNull(ABNEHMERGRP::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Abnehmergruppe '%1' ist als gelscht gekennzeichnet."),
                           BsaId::ABNEHMGRP_NOTFOUND).Arg(req.abnehmergrp);
   }

   ans.bezeichnung =       pB->GetString(ABNEHMERGRP::BEZEICHNUNG);
   ans.nora =              pB->GetBoolNull(ABNEHMERGRP::NORA).Get();
   ans.mikabnehmergrp =    pB->GetString(ABNEHMERGRP::MIKABNEHMERGRUPPE);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetUst(const ReqUst& req, AnsUst& ans, bool incl_ungueltig)
{
   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase>     pB(new BZBoBase(pZD));

   pB->Set(UMSATZSTEUER::USTCODE        , req.ustcode);
   pB->Set(UMSATZSTEUER::GUELTIGVONDATUM, req.vondatum);
   pB->Set(UMSATZSTEUER::INCLUNGUELTIG  , incl_ungueltig);   //XSW 22.6.2011 bei true werden auch ungltige Records gelesen

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Eintrag in Umsatzsteuer nicht gefunden. Ustcode: %1"),
                   BsaId::UMSATZSTEUER_NOTFOUND).Arg(req.ustcode);
      }
      else
         throw;
   }

   if (!req.stbhandlustlfdnr.IsNull())
   {
      BZBoBase oBoStBehUst(new STBEHANDLUST());

      oBoStBehUst.Set(STBEHANDLUST::LFDNR  , req.stbhandlustlfdnr);
      oBoStBehUst.Set(STBEHANDLUST::USTCODE, req.ustcode         );

      try
      {
         oBoStBehUst.Read();

         if (oBoStBehUst.IsNull(STBEHANDLUST::UVACODE))
         {
            BSTHROW BSException(i18n("Bei Steuerlichen Behandlung '%1' und Ustcode '%2' ist kein UVA-Code eingetragen"))
                    .Arg(req.stbhandlustlfdnr).Arg(req.ustcode);
         }
         ans.uvacode = oBoStBehUst.Get(STBEHANDLUST::UVACODE);
      }
      catch(BZException& oBZException)
      {
         if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
         {
            if (pB->Get(UMSATZSTEUER::ISSTEUERBAR).Get(false))
            {
               BSTHROW BSException(i18n("Bei Steuerlichen Behandlung '%1' ist Ustcode '%2' nicht definiert"))
                        .Arg(req.stbhandlustlfdnr).Arg(req.ustcode);
            }
            else
            {
               ans.uvacode = L"ATXXX";
            }
         }
         else
            throw;
      }
   }

   ans.ustprozsatz    = pB->Get(UMSATZSTEUER::USTSATZ);          //NEU
   ans.ustcodegigwl   = pB->Get(UMSATZSTEUER::USTCODEGIGWL);
   ans.ustcodegiteile = pB->Get(UMSATZSTEUER::USTCODEGITEILE);
   ans.isdefault      = pB->Get(UMSATZSTEUER::ISDEFAULT).Get();
   ans.issteuerbar    = pB->Get(UMSATZSTEUER::ISSTEUERBAR).Get();
   ans.drucktext      = pB->Get(UMSATZSTEUER::DRUCKTEXT);
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetUstDiff(AnsUst& ans)
{
   ans.ustprozsatz      = 0.0;
   ans.ustcodegigwl     = L"";
   ans.ustcodegiteile   = L"";
   ans.issteuerbar      = false;
   ans.isdefault        = false;
   ans.drucktext        = L"";
   ans.uvacode          = L"ATXXX";
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetUstCode(const ReqUstCode& req, AnsUstCode& ans)
{
   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new UMSATZSTEUER::ListeUstCode);
   BZBoList boList(pB, pList);

   boList.Param()->SetDoubleNull(UMSATZSTEUER::ListeUstCode::Param::SATZ, req.ustsatz);
   boList.Param()->SetDate(UMSATZSTEUER::ListeUstCode::Param::DATUM, req.datum);
   boList.Param()->SetBoolNull(UMSATZSTEUER::ListeUstCode::Param::DEFAULTCODE, req.defaultCode);

   //Nullsatz finden; wenn req.ustsatz=NULL wird im ZAD nmlich das kriterium ausgeschlossen =>
   //AND ustsatz = :satz  ? !p.satz.IsNull()
   if(req.ustsatz.IsNull())
      boList.Param()->SetBoolNull(UMSATZSTEUER::ListeUstCode::Param::NULLUSTSATZ, true);

   boList.Read();

   // es darf nur ein gltiger Prozentsatz zurckkommen ...
   if (boList.Count() > 1)
      BSTHROW BZException(i18n("Mehr als ein Umsatzsteuersatz %1, %2"),
                           BsaId::UMSATZSTEUER_NOTFOUND).Arg(req.ustsatz.Get(0)).Arg(req.defaultCode.Get(false));

   if (boList.Count() < 1)
   {
      if(req.ustsatz.IsNull())
         BSTHROW BZException(i18n("Eintrag in Umsatzsteuer nicht gefunden. Ustsatz=NULL"),BsaId::UMSATZSTEUER_NOTFOUND);
      else
         BSTHROW BZException(i18n("Eintrag in Umsatzsteuer nicht gefunden. Ustsatz: %1, IsDefault: %2"),
                             BsaId::UMSATZSTEUER_NOTFOUND).Arg(req.ustsatz.Get()).Arg(req.defaultCode.Get(false));
   }

   ans.ustcode = boList.GetBO(0)->GetString(UMSATZSTEUER::USTCODE);
   ans.ustcodegigwl = boList.GetBO(0)->GetString(UMSATZSTEUER::USTCODEGIGWL);
}

////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetUstCodeNichtSteuerbar(bool &moreThanOneExists)   
{
   moreThanOneExists=false;

   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new UMSATZSTEUER::SuppUst);
   BZBoList boList(pB, pList);

   boList.Param()->SetDate(UMSATZSTEUER::SuppUst::Param::GUELTIGDATUM, BSDate::CurrentDate());
   boList.Param()->SetBool(UMSATZSTEUER::SuppUst::Param::ISSTEUERBAR_EQ, false);
   boList.Read();

   int pos=0;
   if(boList.Count() >= 1)
   {
      moreThanOneExists=true;
      for(int i=0; i<boList.Count(); i++)
      {
         if(boList.GetBO(i)->GetBoolNull(UMSATZSTEUER::ISDEFAULT).Get())
         {
            pos=i;
            break;
         }
      }

      return boList.GetBO(pos)->GetString(UMSATZSTEUER::USTCODE);
   }
   else
      BSTHROW BSException(i18n("Kein gltiger Umsatzsteuercode fr 'nicht steuerbar' gefunden!"));
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetUstGiTeile(const CString& ustCodeGiTeile, CString& ustcode)
{
   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new UMSATZSTEUER::Liste);
   BZBoList boList(pB, pList);
   boList.Read();

   int index = boList.GetIndex(UMSATZSTEUER::USTCODEGITEILE, ustCodeGiTeile);

   if (index < 0)
      BSTHROW BZException(i18n("Eintrag in Umsatzsteuer nicht gefunden. UstcodeGITeil: %1"),
                              BsaId::UMSATZSTEUER_NOTFOUND).Arg(ustCodeGiTeile);            

   ustcode = boList.GetBO(index)->GetString(UMSATZSTEUER::USTCODE);
}


////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::IsValidUstSteuerlBeh(const ReqUstSteuerlBeh& req)
{
   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new UMSATZSTEUER::ListeValidStBeh);
   BZBoList boList(pB, pList);

   boList.Param()->SetString(UMSATZSTEUER::ListeValidStBeh::Param::CODE, req.ustcode);
   boList.Param()->SetDate(UMSATZSTEUER::ListeValidStBeh::Param::DATUM, req.vondatum);
   boList.Param()->SetLong(UMSATZSTEUER::ListeValidStBeh::Param::STEUERBH, req.steuerbh.Get());

   boList.Read();

   if (boList.Count() > 0)
      return true;

   return false;
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetStandardwerte(AnsStdwAllgemein& ans)
{
   BSPtr<ZDDataStruct> pZD(new STANDARDWERTE);
   BSPtr<BZBoBase>     pB (new BZBoBase(pZD));
   pB->SetLong(STANDARDWERTE::LFDNR, 1L);
   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Allgemeine Standardwerte nicht gefunden."),
            BsaId::STDWERTE_NOTFOUND);

      }
      else
         throw;
   }

   ans.ustcode           = pB->GetString  (STANDARDWERTE::USTCODE);
   ans.steuerlbehverk    = pB->GetLongNull(STANDARDWERTE::STEUERLBEH);
   ans.steuerlbehustverk = pB->GetString  (STANDARDWERTE::STEUERLBEHUST);
   ans.steuerlbeheink    = pB->GetLongNull(STANDARDWERTE::STEUERLBEHEINK);
   ans.steuerlbehusteink = pB->GetString  (STANDARDWERTE::STEUERLBEHUSTEINK);
   ans.zuhanden          = pB->GetString  (STANDARDWERTE::ZUHANDEN);
}

void MgrBsa::GetStandardwerte(int mandantnr, AnsStdwMandant& ans)
{
   BSPtr<ZDDataStruct> pZD  (new STDWMANDANT);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));

   pB->SetInt(STDWMANDANT::MANDANTNR, mandantnr);
   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Standardwerte fr Mandant %1 nicht gefunden."),
            BsaId::STDWERTE_NOTFOUND).Arg(mandantnr);

      }
      else
         throw;
   }

   ans.zahlungsmittel   = pB->GetLongNull(STDWMANDANT::ZAHLUNGSMITTEL);
   ans.zahlungsziel     = pB->GetLongNull(STDWMANDANT::ZAHLUNGSZIEL);
   ans.umsatz_von_monat = pB->GetIntNull (STDWMANDANT::UMSATZZEITRVON);
   ans.wsabnehmergrp    = pB->GetString  (STDWMANDANT::WSABNEHMERGRP);
   ans.nwabnehmergrp    = pB->GetString  (STDWMANDANT::NWABNEHMERGRP);
   ans.gwabnehmergrp    = pB->GetString  (STDWMANDANT::GWABNEHMERGRP);
   ans.nwzahlungsziel   = pB->GetLongNull(STDWMANDANT::NWZAHLUNGSZIEL);
   ans.gwzahlungsziel   = pB->GetLongNull(STDWMANDANT::GWZAHLUNGSZIEL);

   GetStandardwerte(ans);
}

void MgrBsa::GetStandardwerte(const MModul& req, AnsStdwArbBereich& ans)
{
   BSPtr<ZDDataStruct> pZD  (new STDWARBBEREICH);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));

   pB->SetInt(STDWARBBEREICH::MANDANTNR, req.mandantnr.Get());
   pB->SetInt(STDWARBBEREICH::MODULNR  , req.modulnr.Get());
   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Standardwerte fr Mandant %1, Modul %2  nicht gefunden."),
            BsaId::STDWERTE_NOTFOUND).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
      }
      else
         throw;
   }

   ans.gptnrstdwerkstatt          = pB->Get(STDWARBBEREICH::GPTNRWERKSTATT            );
   ans.par57text1                 = pB->Get(STDWARBBEREICH::PAR57ERINNTEXT1           );
   ans.par57text2                 = pB->Get(STDWARBBEREICH::PAR57ERINNTEXT2           );
   ans.par57text3                 = pB->Get(STDWARBBEREICH::PAR57ERINNTEXT3           );
   ans.datenschutzerklaerungDruck = pB->Get(STDWARBBEREICH::DATENSCHUTZERKLAERUNGDRUCK);
   ans.eMailBetreff               = pB->Get(STDWARBBEREICH::EMAILBETREFF              );
   ans.eMailText                  = pB->Get(STDWARBBEREICH::EMAILTEXT                 );

   GetStandardwerte(req.mandantnr.Get(-1), ans);
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetDefaultIntFzgArt(const CString& zweck, AnsIntFzgArt& ans, bool incl_deleted)
{
   BSPtr<BZBoBase>     pB   (new BoIntFzgArt);
   BSPtr<ZDListAccess> pList(new INTFZGART::Liste);
   BZBoList bolist(pB, pList);

   bolist.Param()->SetBool(INTFZGART::Liste::Param::ISDEFAULT_EQ, true);
   bolist.Param()->SetString(INTFZGART::Liste::Param::ZWECK_EQ, zweck);

   bolist.Read();

   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);
      if(!incl_deleted && pBo->GetBoolNull(INTFZGART::LOESCHKZ).Get())
         continue;

      MapIntFzgArt(*pBo, ans);
      return;
   }

   BSTHROW BZException(i18n("Keine Interne Fahrzeugart fr den Zweck '%1' als Standard definiert"),
                           BsaId::INTFZGART_NOTFOUND).Arg(zweck);
}

void MgrBsa::GetDefaultIntFzgArtHerkGrpZuord(const CString &sInterneFzgArt, const CString &sZweck, CString &sHerkunftsgruppe) const
{
   BZBoBase oBoHerkunftsgruppe(new INTFZGARTHERKGRPZUORD());
   oBoHerkunftsgruppe.Set(INTFZGARTHERKGRPZUORD::INTERNEFZGART, sInterneFzgArt);
   oBoHerkunftsgruppe.Set(INTFZGARTHERKGRPZUORD::ZWECK, sZweck);

   BSPtr<ZDMultiAccess> pSpecAccess(new INTFZGARTHERKGRPZUORD::GetStandardHerkGrp());
   if (oBoHerkunftsgruppe.ReadIfExists(pSpecAccess))
      sHerkunftsgruppe = oBoHerkunftsgruppe.Get(INTFZGARTHERKGRPZUORD::HERKUNFTSGRUPPE);
   else
      sHerkunftsgruppe.Empty();
}

void MgrBsa::GetIntFzgArt(const ReqIntFzgArt& req, AnsIntFzgArt& ans, bool incl_deleted)
{
   BSPtr<BZBoBase>     pB   (new BoIntFzgArt);
   pB->SetString(INTFZGART::ZWECK, req.zweck);
   pB->SetString(INTFZGART::INTERNEFZGART, req.intfzgart);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Interne Fahrzeugart '%1'/'%2' nicht gefunden"),
                              BsaId::INTFZGART_NOTFOUND).Arg(req.intfzgart)
                                                               .Arg(req.zweck);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(INTFZGART::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Interne Fahrzeugart '%1'/'%2' ist als gelscht gekennzeichnet."),
                           BsaId::INTFZGART_NOTFOUND).Arg(req.intfzgart)
                                                            .Arg(req.zweck);
   }
   MapIntFzgArt(*pB, ans);
}

void MgrBsa::GetIntFzgArtListe(const CString&         zweck,
                               BSTCont<AnsIntFzgArt>& ans  ,
                               bool                   incl_deleted/*=false*/)
{
   ans.Clear();

   BSPtr<BZBoBase>     pB   (new BoIntFzgArt);
   BSPtr<ZDListAccess> pList(new INTFZGART::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(INTFZGART::Liste::Param::ZWECK_EQ, zweck);
   bolist.Read();

   AnsIntFzgArt* ansart;
   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);
      if(pBo->GetString(INTFZGART::ZWECK) != zweck)
         continue;
      if(!incl_deleted && pBo->GetBoolNull(INTFZGART::LOESCHKZ).Get())
         continue;

      ansart = new AnsIntFzgArt;
      ans.Insert(ansart);
      MapIntFzgArt(*pBo, *ansart);
   }
}

void MgrBsa::MapIntFzgArt(const BZBoBase& bo, AnsIntFzgArt& ans) const
{
   ans.zweck             = bo.GetString  (INTFZGART::ZWECK);
   ans.intfzgart         = bo.GetString  (INTFZGART::INTERNEFZGART);
   ans.inventur          = bo.GetBoolNull(INTFZGART::INVENTUR).Get();
   ans.folgeakterstellen = bo.GetBoolNull(INTFZGART::FOLGEAKTERSTELLEN).Get();
   ans.stilllegung       = bo.GetBoolNull(INTFZGART::STILLLEGUNG).Get();
}

///////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetIntGFTWE(const ReqIntGF& req, AnsIntGFTWE& ans, bool incl_deleted)
{
   BSPtr<BZBoBase> pB(new BoTWEIntGF);
   pB->SetInt (TWEINTGF::MANDANTNR , req.mandantnr.Get());
   pB->SetInt (TWEINTGF::MODULNR   , req.modulnr.Get());
   pB->SetLong(TWEINTGF::LFDNR     , req.lfdnr.Get());

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Interner TWE Geschftsfall '%1' fr Mandant %2, Modul %3 nicht gefunden"),
                              BsaId::TWEINTGF_NOTFOUND)
                              .Arg(req.lfdnr.Get()).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(TWEINTGF::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Interner TWE Geschftsfall '%1' fr Mandant %2, Modul %3 ist als gelscht gekennzeichnet."),
                           BsaId::TWEINTGF_NOTFOUND)
                           .Arg(req.lfdnr.Get()).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
   }

   ans.kurzbezeichnung    = pB->GetString   (TWEINTGF::KURZBEZ         );
   ans.anrede             = pB->GetString   (TWEINTGF::ANREDE          );
   ans.bezeichnung1       = pB->GetString   (TWEINTGF::BEZEICHNUNG1    );
   ans.bezeichnung2       = pB->GetString   (TWEINTGF::BEZEICHNUNG2    );
   ans.kostenstelle       = pB->GetString   (TWEINTGF::KOSTENSTELLE    );
   ans.abnehmergruppe     = pB->GetString   (TWEINTGF::ABNEHMERGRUPPE  );
   ans.mwstkz             = pB->GetString   (TWEINTGF::MWSTKZ          );
   ans.sachkonto          = pB->GetString   (TWEINTGF::SACHKONTO       );
   ans.steuerlbeh         = pB->GetLongNull (TWEINTGF::STEUERLBEH      ).Get();
   ans.fzgerforderl       = pB->GetBoolNull (TWEINTGF::FZGERFORDERL    ).Get();
   ans.betreffverlangen   = pB->GetBoolNull (TWEINTGF::BETREFFVERLANGEN).Get();
   ans.is_deleted         = pB->GetBoolNull (TWEINTGF::LOESCHKZ        ).Get();
   ans.vorsteuerkorrektur = pB->GetBoolNull (TWEINTGF::VSTKORR         ).Get(false);
   ans.discencode         = pB->GetString   (TWEINTGF::DISCENCODE      );
   ans.zusatzmerkmal      = pB->GetString   (TWEINTGF::ZUSATZMERKMAL   );
   ans.kostentraeger      = pB->GetString   (TWEINTGF::KOSTENTRAEGER   );
   ans.koremerkmalebeiumbuch = pB->GetBoolNull(TWEINTGF::KOREMERKMALEBEIUMBUCH).Get();
   ans.elektrrechnung     = pB->GetBoolNull (TWEINTGF::ELEKTRRECHNUNG  ).Get();
}

void MgrBsa::GetIntGFNGE(const ReqIntGF& req, AnsIntGFNGE& ans, bool incl_deleted)
{
   BSASSERT(!req.lfdnr.IsNull());

   BSPtr<ZDDataStruct> pZD  (new NGEGF);
   BSPtr<ZDListAccess> pList(new NGEGF::AccListNgegf18);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BZBoList bolist(pB, pList);
   bolist.Param()->SetInt(   NGEGF::AccListNgegf18::Param::MANDANTNR_EQ,  req.mandantnr.Get(-1));
   bolist.Param()->SetInt(   NGEGF::AccListNgegf18::Param::MODULNR_EQ  ,  req.modulnr.Get(-1)  );
   bolist.Param()->SetLong(  NGEGF::AccListNgegf18::Param::NGEGFLFDNR_EQ, req.lfdnr.Get()      );
   bolist.Param()->SetString(NGEGF::AccListNgegf18::Param::ZWECK_EQ,      req.zweck            );

   bool bFound = false;
   const CString sFzgZweckIntern = csIni->GetValue(L"INT", L"INTGFZWECK", &bFound, L"");
   bolist.Param()->SetString(NGEGF::AccListNgegf18::Param::ZWECK_FZG_INTERN_EQ, sFzgZweckIntern);

   bolist.Read();

   if (bolist.Count() <= 0)
   {
      BSTHROW BZException(i18n("NGS Geschftsfall '%1' fr Mandant %2, Modul %3 nicht gefunden"),
                           BsaId::NGEINTGF_NOTFOUND)
                           .Arg(req.lfdnr.Get()).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
   }

   const BZBoBase* pBo  = bolist.GetBO(0);

   if (!incl_deleted && 
       (pBo->GetBoolNull(NGEGF::LOESCHKZ        ).Get() ||
        pBo->GetBoolNull(NGEGF::NGEGFARBLOESCHKZ).Get() ))
   {
      BSTHROW BZException(i18n("NGS Geschftsfall '%1' (%2) fr Mandant %3, Modul %4 ist als gelscht gekennzeichnet."),
                           BsaId::NGEINTGF_NOTFOUND)
                           .Arg(pBo->GetString(NGEGF::BEZEICHNUNG)).Arg(req.lfdnr.Get()).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
   }

   MapIntGFNGE(*pBo, ans);
   ans.gf_found = true;
}

void MgrBsa::GetListeIntGFNGE(const MModul& req, BSTCont<AnsIntGFNGE>& ans, bool incl_deleted)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD  (new NGEGF);
   BSPtr<ZDListAccess> pList(new NGEGF::AccListNgegf18);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BZBoList bolist(pB, pList);
   bolist.Param()->SetInt(NGEGF::AccListNgegf18::Param::MANDANTNR_EQ, req.mandantnr.Get(-1));
   bolist.Param()->SetInt(NGEGF::AccListNgegf18::Param::MODULNR_EQ  , req.modulnr.Get(-1)  );

   bool bFound = false;
   const CString sFzgZweckIntern = csIni->GetValue(L"INT", L"INTGFZWECK", &bFound, L"");
   bolist.Param()->SetString(NGEGF::AccListNgegf18::Param::ZWECK_FZG_INTERN_EQ, sFzgZweckIntern);

   bolist.Read();

   for(int iter=0; iter<bolist.Count(); iter++)
   {
      BZBoBase* pBo = bolist.GetBO(iter);

      // Als gelscht markierte Eintrge auf Wunsch nicht liefern
      if (!incl_deleted && 
          (pBo->GetBoolNull(NGEGF::LOESCHKZ        ).Get() ||
           pBo->GetBoolNull(NGEGF::NGEGFARBLOESCHKZ).Get() ))
         continue;

      BSPtr<AnsIntGFNGE> pItem(new AnsIntGFNGE());
      MapIntGFNGE(*pBo, *pItem);
      ans.Insert(pItem.release());
   }
}

void MgrBsa::GetIntGFNGE(AnsIntGFNGE& ans, const MModul& req, const CString& zweck)
{
   BSPtr<ZDDataStruct> pZD  (new NGEGF);
   BSPtr<ZDListAccess> pList(new NGEGF::AccListNgegf14);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BZBoList bolist(pB, pList);
   bolist.Param()->SetInt(   NGEGF::AccListNgegf14::Param::MANDANTNR_EQ, req.mandantnr.Get(-1));
   bolist.Param()->SetInt(   NGEGF::AccListNgegf14::Param::MODULNR_EQ  , req.modulnr.Get(-1)  );
   bolist.Param()->SetString(NGEGF::AccListNgegf14::Param::ZWECK1_EQ   , zweck                );

   bool bFound = false;
   const CString sFzgZweckIntern = csIni->GetValue(L"INT", L"INTGFZWECK", &bFound, L"");
   bolist.Param()->SetString(NGEGF::AccListNgegf14::Param::ZWECK_FZG_INTERN_EQ, sFzgZweckIntern);

   bolist.Read();

   if (bolist.Count() <= 0)
   {
      BSTHROW BZException(i18n("NGS Geschftsfall fr Mandant %2, Modul %3 und Zweck '%1' nicht gefunden"),
                           BsaId::NGEINTGF_NOTFOUND)
                           .Arg(zweck).Arg(req.mandantnr.Get(-1)).Arg(req.modulnr.Get(-1));
   }

   const BZBoBase* pBo  = bolist.GetBO(0);
   MapIntGFNGE(*pBo, ans);
   ans.gf_found = true;
}

void MgrBsa::GetIntGFNGESSAttribut(const ReqIntGFSS& req, BSTCont<AnsIntGFSS>& ans)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD  (new NGEGFSS);
   BSPtr<ZDListAccess> pList(new NGEGFSS::Liste);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BZBoList bolist(pB, pList);
   bolist.Param()->SetLong  (NGEGFSS::Liste::Param::NGEGFLFDNR_EQ   , req.lfdnr.Get(-1));
   bolist.Param()->SetString(NGEGFSS::Liste::Param::SCHNITTSTELLE_EQ, req.schnittstelle);
   bolist.Read();
   
   for(int iter = 0; iter < bolist.Count(); iter++)
   {
      BSPtr<AnsIntGFSS> pAns(new AnsIntGFSS());

      pAns->schnittstelle = bolist.GetBO(iter)->GetString(NGEGFSS::SCHNITTSTELLE);
      pAns->ssattribut    = bolist.GetBO(iter)->GetString(NGEGFSS::ATTRIBUT);

      ans.Insert(pAns.release());
   }
}


void MgrBsa::MapIntGFNGE(const BZBoBase& bo, AnsIntGFNGE& ans) const
{
   ans.lfdnr               = bo.Get(NGEGF::NGEGFLFDNR         ).Get();
   ans.kurzbezeichnung     = bo.Get(NGEGF::KURZBEZ            );
   ans.bezeichnung         = bo.Get(NGEGF::BEZEICHNUNG        );
   ans.zweck               = bo.Get(NGEGF::ZWECK              );
   ans.verbuchung          = bo.Get(NGEGF::VERBUCHUNG         );
   ans.ergebniswirksam     = bo.Get(NGEGF::ERGEBNISWIRKSAM    );
   ans.wagentyp            = bo.Get(NGEGF::WAGENTYP           );
   ans.einkverkkz          = bo.Get(NGEGF::EINKVERKKZ         );
   ans.leistungsart        = bo.Get(NGEGF::LEISTUNGSART       );
   ans.stueckwirksam       = bo.Get(NGEGF::STUECKWIRKSAM      ).Get();
   ans.iskalkulatorisch    = bo.Get(NGEGF::ISKALKULATORISCH   ).Get();
   ans.isgwreparatur       = bo.Get(NGEGF::ISGWREPARATUR      ).Get();
   ans.isekbeeinflussend   = bo.Get(NGEGF::ISEKBEEINFLUSSEND  ).Get();
   ans.verbuchen           = bo.Get(NGEGF::VERBUCHEN          ).Get();
   ans.absatzkanalrechempf = bo.Get(NGEGF::ABSATZKANALRECHEMPF).Get();
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetZahlungsmittel(long zahlungsmittel, AnsZahlungsmittel& ans, bool incl_deleted)
{
   BSPtr<BZBoBase> pB(new BoZahlungsmittel);
   pB->SetLong(ZHLGSMITTEL::ZAHLUNGSMITTEL, zahlungsmittel);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Zahlungsmittel '%1' nicht gefunden."),
                              BsaId::ZAHLUNGSMITTEL_NOTFOUND).Arg(zahlungsmittel);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(ZHLGSMITTEL::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Zahlungsmittel '%1' ('%2') ist als gelscht gekennzeichnet."),
                           BsaId::ZAHLUNGSMITTEL_NOTFOUND).Arg(pB->GetString(ZHLGSMITTEL::BEZEICHNUNG)).Arg(zahlungsmittel);
   }

   ans.bezeichnung         = pB->GetString  (ZHLGSMITTEL::BEZEICHNUNG);
   ans.zahlungsart         = pB->GetString  (ZHLGSMITTEL::ZAHLUNGSART);
   ans.bargeldkz           = pB->GetBoolNull(ZHLGSMITTEL::BARGELDKZ).Get(); 
   ans.scheckkz            = pB->GetBoolNull(ZHLGSMITTEL::SCHECKKZ).Get(); 
   ans.einzelzhlgverbkz    = pB->GetBoolNull(ZHLGSMITTEL::EINZELZHLGVERBKZ).Get(); 
   ans.zhlgsmittelrechmeld = pB->GetString  (ZHLGSMITTEL::ZHLGSMITTELRECHMELD);
   ans.steuerlbeh_ueberst  = pB->GetLongNull(ZHLGSMITTEL::STEUERLBEH_UEBERST);
   ans.bhcode              = pB->GetString  (ZHLGSMITTEL::BHCODE);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetZahlungsmittelMap(BSMap<long, IMgrBsa::AnsZahlungsmittel>& oANSMap)
{
   BSPtr<ZDDataStruct> pZD(new ZHLGSMITTEL);
   BSPtr<BZBoBase>     pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ZHLGSMITTEL::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();
   for(int i=0; i<bolist.Count(); i++)
   {
      IMgrBsa::AnsZahlungsmittel* pAns = new IMgrBsa::AnsZahlungsmittel();
      pAns->bezeichnung         = bolist.GetBO(i)->GetString(ZHLGSMITTEL::BEZEICHNUNG);
      pAns->zahlungsart         = bolist.GetBO(i)->GetString(ZHLGSMITTEL::ZAHLUNGSART);
      pAns->bargeldkz           = bolist.GetBO(i)->GetBoolNull(ZHLGSMITTEL::BARGELDKZ).Get();
      pAns->scheckkz            = bolist.GetBO(i)->GetBoolNull(ZHLGSMITTEL::SCHECKKZ).Get();
      pAns->einzelzhlgverbkz    = bolist.GetBO(i)->GetBoolNull(ZHLGSMITTEL::EINZELZHLGVERBKZ).Get();
      pAns->zhlgsmittelrechmeld = bolist.GetBO(i)->GetString(ZHLGSMITTEL::ZHLGSMITTELRECHMELD);
      pAns->steuerlbeh_ueberst  = bolist.GetBO(i)->GetLongNull(ZHLGSMITTEL::STEUERLBEH_UEBERST);
      oANSMap.Insert(bolist.GetBO(i)->GetLongNull(ZHLGSMITTEL::ZAHLUNGSMITTEL).Get(-1), pAns);
   }
}
////////////////////////////////////////////////////////////////////////////////
double MgrBsa::GetUmrechnungsfaktor(const CString& fromWKZ, 
                                        const CString& toWKZ, 
                                        const CString& verrfabrikatkz,
                                        BSDate datum)
{
   if(fromWKZ == toWKZ)
      return 1;

   // diese Liste gibt uns alle BOs zurck ber welche eine
   // Umrechnung mglich whre !!
   BSPtr<BZBoBase>     pB   (new BoWaehrung);
   BSPtr<ZDListAccess> pList(new WAEHRUNG::AccListWaehrung13);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(WAEHRUNG::AccListWaehrung13::Param::WAEHRUNGSKZFROM_EQ, fromWKZ);
   bolist.Param()->SetString(WAEHRUNG::AccListWaehrung13::Param::WAEHRUNGSKZTO_EQ  , toWKZ);
   bolist.Param()->SetString(WAEHRUNG::AccListWaehrung13::Param::VERRFABRIKATKZ_EQ , verrfabrikatkz);
   bolist.Param()->SetDateNull(WAEHRUNG::AccListWaehrung13::Param::DATUM_EQ,         datum);

   bolist.Read();

   double umrechnungsfaktor_direkt_ohneverrfa = 0;
   double umrechnungsfaktor_indirekt_mitverrfa = 0;
   double umrechnungsfaktor_indirekt_ohneverrfa = 0; 

   if(bolist.Count() <= 0)
      BSTHROW BZException(i18n("Es wurde kein gltiger Umrechnungsfaktor von "
                                "'%1' auf '%2' fr das Fabrikatskennzeichen '%3' gefunden"),
                           BsaId::UMRECHFAK_NOTFOUND)
                           .Arg(fromWKZ).Arg(toWKZ).Arg(verrfabrikatkz);

   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);

      if(  (   pBo->GetString(WAEHRUNG::WAEHRUNGSKZ1) == fromWKZ
            && pBo->GetString(WAEHRUNG::WAEHRUNGSKZ2) == toWKZ   )
         ||(   pBo->GetString(WAEHRUNG::WAEHRUNGSKZ1) == toWKZ
            && pBo->GetString(WAEHRUNG::WAEHRUNGSKZ2) == fromWKZ   )   )
      {
         if(   pBo->GetString(WAEHRUNG::VERRFABRIKATKZ1) == verrfabrikatkz
            && pBo->GetString(WAEHRUNG::VERRFABRIKATKZ2) == verrfabrikatkz )
         {
            return pBo->GetDoubleNull(WAEHRUNG::UMRECHNUNGSFAKTOR).Get();
         }
         else
            umrechnungsfaktor_direkt_ohneverrfa = pBo->GetDoubleNull(WAEHRUNG::UMRECHNUNGSFAKTOR).Get();
      }
      else
      {
         if(   pBo->GetString(WAEHRUNG::VERRFABRIKATKZ1) == verrfabrikatkz
            && pBo->GetString(WAEHRUNG::VERRFABRIKATKZ2) == verrfabrikatkz )
         {
            umrechnungsfaktor_indirekt_mitverrfa = pBo->GetDoubleNull(WAEHRUNG::UMRECHNUNGSFAKTOR).Get();
         }
         else
            umrechnungsfaktor_indirekt_ohneverrfa = pBo->GetDoubleNull(WAEHRUNG::UMRECHNUNGSFAKTOR).Get();
      }
   }
      
   double umrechnungsfaktor = 0;

   if(umrechnungsfaktor_indirekt_mitverrfa > 0)
      umrechnungsfaktor =  umrechnungsfaktor_indirekt_mitverrfa;

   else if(umrechnungsfaktor_direkt_ohneverrfa > 0)
      umrechnungsfaktor =  umrechnungsfaktor_direkt_ohneverrfa;

   else
      umrechnungsfaktor =  umrechnungsfaktor_indirekt_ohneverrfa;
   return umrechnungsfaktor;
}

double MgrBsa::GetUmrechnungsfaktorHWtoZW()
{
   CString hauswaehrung  = SOIni::Instance()->GetValue(L"HAUSWAEHRUNG" ,L"WAEHRUNG");
   CString zweitwaehrung = SOIni::Instance()->GetValue(L"ZWEITWAEHRUNG",L"WAEHRUNG");

   if(hauswaehrung == zweitwaehrung)
      return 1;

   // Umrechnungsfunktionalitt aktiv?
   if(SOIni::Instance()->GetValue(L"UMRECHNUNG_ZWEITWAEHRUNG",L"EINSTELLUNGEN") != L"J")
      return 1;

   return GetUmrechnungsfaktor(hauswaehrung, zweitwaehrung, L""); //verrfab muss fr alle gelten
}

double_null MgrBsa::CalcHauswaehrung(const double_null& Zweitwaehrung, bool runden)
{
   if(Zweitwaehrung.IsNull())
      return double_null();

   // Wenn beide Waehrungseinheiten gleich sind erfolgt keine Umrechnung:
   // WEJ26.02.2002
   if(SOIni::Instance()->GetValue(L"HAUSWAEHRUNG" ,L"WAEHRUNG") ==
      SOIni::Instance()->GetValue(L"ZWEITWAEHRUNG",L"WAEHRUNG") )  
      return Zweitwaehrung;

   // Prfung, ob Umrechnungsfunktionalitt aktiv, in GetUmrechnungsfaktorHWtoZW:
   double umrechnungsfaktor = (1 / GetUmrechnungsfaktorHWtoZW());

   if(runden)
      return BSRound(Zweitwaehrung.Get() * umrechnungsfaktor, 2);
   else
      return double_null(Zweitwaehrung.Get() * umrechnungsfaktor);
}

double_null MgrBsa::CalcZweitwaehrung(const double_null& Hauswaehrung, bool runden)
{
   if(Hauswaehrung.IsNull())
      return double_null();

   // Wenn beide Waehrungseinheiten gleich sind erfolgt keine Umrechnung:
   if(SOIni::Instance()->GetValue(L"HAUSWAEHRUNG" ,L"WAEHRUNG") ==
      SOIni::Instance()->GetValue(L"ZWEITWAEHRUNG",L"WAEHRUNG") )  
      return Hauswaehrung;

   // Prfung, ob Umrechnungsfunktionalitt aktiv, in GetUmrechnungsfaktorHWtoZW:
   double umrechnungsfaktor = GetUmrechnungsfaktorHWtoZW();

   if(runden)
      return BSRound(Hauswaehrung.Get() * umrechnungsfaktor, 2);
   else
      return double_null(Hauswaehrung.Get() * umrechnungsfaktor);
}

void MgrBsa::CalcHauswaehrungFromFremd(const ReqWaehrung& req, double_null& Hauswaehrung)
{
   CString Hauswaehrungskz = SOIni::Instance()->GetValue(L"HAUSWAEHRUNG" ,L"WAEHRUNG");

   if(req.fromBetrag.IsNull())
   {
      Hauswaehrung.SetNull();
      return;
   }

   if(Hauswaehrungskz == req.fromWaehrung)
   {
      Hauswaehrung = req.fromBetrag;
      return;
   }

   double umrechnungsfaktor;
   if(req.gueltig_datum.IsNull())
      umrechnungsfaktor = GetUmrechnungsfaktor(req.fromWaehrung, Hauswaehrungskz, 
                                               req.verrfabrikatkz);
   else
      umrechnungsfaktor = GetUmrechnungsfaktor(req.fromWaehrung, Hauswaehrungskz, 
                                               req.verrfabrikatkz, req.gueltig_datum.Get());

   if(req.runden.Get(false))
      Hauswaehrung = BSRound(req.fromBetrag.Get() * umrechnungsfaktor, 2);
   else
      Hauswaehrung = (req.fromBetrag.Get() * umrechnungsfaktor);
}

void MgrBsa::CalcZweitwaehrungFromFremd(const ReqWaehrung& req, double_null& ZweitWaehrung)
{
   CString zweitwaehrungskz = SOIni::Instance()->GetValue(L"ZWEITWAEHRUNG",L"WAEHRUNG");

   if(req.fromBetrag.IsNull())
   {
      ZweitWaehrung.SetNull();
      return;
   }

   if(zweitwaehrungskz == req.fromWaehrung)
   {
      ZweitWaehrung = req.fromBetrag;
      return;
   }

   double umrechnungsfaktor;
   if(req.gueltig_datum.IsNull())
      umrechnungsfaktor = GetUmrechnungsfaktor(req.fromWaehrung, zweitwaehrungskz, 
                                               req.verrfabrikatkz);
   else
      umrechnungsfaktor = GetUmrechnungsfaktor(req.fromWaehrung, zweitwaehrungskz, 
                                               req.verrfabrikatkz, req.gueltig_datum.Get());

   if(req.runden.Get(false))
      ZweitWaehrung = BSRound( req.fromBetrag.Get() * umrechnungsfaktor, 2);
   else
      ZweitWaehrung = (req.fromBetrag.Get() * umrechnungsfaktor);
}

////////////////////////////////////////////////////////////////////////////////
long MgrBsa::GetDBId()
{
   BSPtr<ZDMultiAccess> pAcc(new BETRIEBDB::ReadDBId());
   BSPtr<BZBoBase> pBo(new BZBoBase(new BETRIEBDB));

   BSASSERT(pAcc.get());
   try
   {
      pBo->Read(pAcc);
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Betriebdb nicht gefunden"),
                              BsaId::BETRIEBDB_NOTFOUND);
      }
      else
         throw;
   }
   return pBo->GetLongNull(BETRIEBDB::DBID).Get();
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetBetriebdbDaten(const CString& betnr, ANSBetriebdb& oANS)
{
   BSPtr<ZDDataStruct> pZD  (new BETRIEBDB);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   pB->SetString  (BETRIEBDB::BETNR, betnr);
   pB->SetLongNull(BETRIEBDB::VPDID_, GetVPDID());

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Betnr '%1' nicht gefunden"),
                              BsaId::BETRIEBDB_NOTFOUND).Arg(betnr);
      }
      else
         throw;
   }
   oANS.betnr        = pB->GetString  (BETRIEBDB::BETNR);
   oANS.dbid         = pB->GetLongNull(BETRIEBDB::DBID);
   oANS.vpdid        = int(pB->GetLongNull(BETRIEBDB::VPDID_).Get(0));
   oANS.beschreibung = pB->GetString  (BETRIEBDB::BESCHREIBUNG);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::CreateBetriebdbEntry(const CString& betnr, bool isArbeitsbereich)
{
   BSASSERT(0);
}
void MgrBsa::CreateBetriebdbEntry(const CString& betnr, const long& gptnr, bool isArbeitsbereich)
{
   //BSLog(BS_TRACE, "MgrBsa::%1:").Arg("CreateBetriebdbEntry");
   BSASSERT(!betnr.IsEmpty());

   // isArbeitsbereich==true:  Auruf der Funktion von Wartung Arbeitsbereiche
   // isArbeitsbereich==false: Auruf der Funktion von GP-Rolle

   if(!isArbeitsbereich)
   {
      BSTCont<AnsArbBereich> ansarb;
      GetArbeitsbereichListe(ansarb,int_null(),L"",betnr);

      // Falls Betrieb NICHT in der BETMANDMODUL 
      // vorhanden ist, dann mu kein Satz in der BETRIEBDB gespeichert werden:
      if(ansarb.Count() <= 0)
         return;
   }

   BSPtr<ZDDataStruct> pZD  (new BETRIEBDB);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new BETRIEBDB::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();

   long_null iVPDID = GetVPDID();
   // ist schon ein Eintrag in der BETRIEBDB mit der betnr vorhanden
   // -> bin schon fertig (mu kein neuer Eintrag mehr gemacht werden)
   if(bolist.GetIndex(BETRIEBDB::VPDID_, iVPDID.Get(0),
                      BETRIEBDB::BETNR , betnr) >= 0)
      return;

   // sonst neuen Eintrag in BETRIEBDB mit der betnr erzeugen
   int index = bolist.GetIndex(BETRIEBDB::VPDID_, iVPDID.Get(0));
   if(index < 0)
      BSTHROW BZException(i18n("Kein Betriebdb mit VPDID '%1' vorhanden"),
                           BsaId::BETRIEBDB_NOTFOUND).Arg(iVPDID);

   const BZBoBase* pBo  = bolist.GetBO(index);

   BSPtr<ZDDataStruct> pZDNew(new BETRIEBDB);
   BSPtr<BZBoBase>     pBoNew(new BZBoBase(pZDNew));
   pBoNew->SetState   (BZBoBase::ISNEW);
   pBoNew->SetString  (BETRIEBDB::BETNR        , betnr);
   pBoNew->SetLongNull(BETRIEBDB::VPDID_       , iVPDID);
   pBoNew->SetLong    (BETRIEBDB::DBID         , pBo->GetLongNull (BETRIEBDB::DBID        ).Get());
   pBoNew->SetString  (BETRIEBDB::BESCHREIBUNG , pBo->GetString  (BETRIEBDB::BESCHREIBUNG));
   pBoNew->Persist();
   bolist.Read(true);

   if(isArbeitsbereich)
   {
      // Fr alle dem Betrieb (betnr) zugeordneten Betriebe (ber gptnr) in der NGSMARKEBETNR 
      // in BETRIEBDB eintragen:

      BSPtr<ZDDataStruct> tmpStrArb	(new ARBBEREICH());
      BSPtr<BZBoBase    > tmpBOArb		(new BZBoBase(tmpStrArb)        );
      BSPtr<ZDListAccess> tmplistAcc	(new ARBBEREICH::ListeMarkebetnr());
      BZBoList tmpBOList(tmpBOArb, tmplistAcc);

      tmpBOList.Param()->SetLongNull(ARBBEREICH::ListeMarkebetnr::Param::GPTNR_EQ, gptnr);
      tmpBOList.Read();

      CString betrieb;
      for(int iter=0; iter<tmpBOList.Count(); iter++)
      {
         betrieb = tmpBOList.GetBO(iter)->GetString(ARBBEREICH::BETNR);
         // Betrieb bereits eingetragen?
         if(bolist.GetIndex(BETRIEBDB::VPDID_, iVPDID.Get(0),
                            BETRIEBDB::BETNR , betrieb) >= 0)
            continue;

         pBoNew->SetState (BZBoBase::ISNEW);
         pBoNew->SetString(BETRIEBDB::BETNR, betrieb);
         pBoNew->Persist();
         bolist.Read(true);
      }
   }
}

void MgrBsa::DeleteBetriebdbEntry(const CString& betnr, bool isArbeitsbereich)
{
   BSASSERT(0);
}
void MgrBsa::DeleteBetriebdbEntry(const CString& betnr, const long& gptnr, bool isArbeitsbereich)
{
   //BSLog(BS_TRACE, "MgrBsa::%1:").Arg("DeleteBetriebdbEntry");
   BSASSERT(!betnr.IsEmpty());

   // Als Erstes mu geprft werden wieviele Stze mit dem dbnamen
   // in der Betriebd vorhanden sind. Falls es nur noch einen 
   // Eintrag mit dem dbnamen gibt -> return true (letzter Eintrag darf 
   // nicht gelscht werden!).
   long_null iVPDID = GetVPDID();

   BSPtr<ZDDataStruct> pZD  (new BETRIEBDB);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new BETRIEBDB::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();

   int dbcount = 0;
   for(int i=bolist.Count()-1; i>=0; i--)
   {
      if(bolist.GetBO(i)->GetLongNull(BETRIEBDB::VPDID_) == iVPDID)
         dbcount++;
   }

   if(dbcount <= 1)
      return;

   // Auruf der Funktion von Wartung Arbeitsbereiche
   if(isArbeitsbereich)
   {
      // Falls noch mehr Eintrge mit der selben betnr in
      // BETMANDMODUL existieren -> return true (Nichts aus BETRIEBDB lschen)
      BSTCont<AnsArbBereich> ansarb;
      GetArbeitsbereichListe(ansarb,int_null(),L"",betnr);
      if(ansarb.Count() > 1)
         return;

      // wenn es der letzte Satz mit der betnr in BETMANDMODUL ist dann...
      if(ansarb.Count() == 1)
      {
         // ...Satz aus BETRIEBDB lschen
         LoescheBetriebdbEntry(betnr);

         BSPtr<ZDDataStruct> tmpStrArb	(new ARBBEREICH());
         BSPtr<BZBoBase    > tmpBOArb		(new BZBoBase(tmpStrArb)        );
         BSPtr<ZDListAccess> tmplistAcc	(new ARBBEREICH::ListeMarkebetnr());
         BZBoList tmpBOList(tmpBOArb, tmplistAcc);

         tmpBOList.Param()->SetLongNull(ARBBEREICH::ListeMarkebetnr::Param::GPTNR_EQ, gptnr);
         tmpBOList.Read();

         CString betrieb;
         for(int iter=0; iter<tmpBOList.Count(); iter++)
         {
            betrieb = tmpBOList.GetBO(iter)->GetString(ARBBEREICH::BETNR);
            ansarb.Clear();
            GetArbeitsbereichListe(ansarb,int_null(),L"",betrieb);
            if(ansarb.Count() >= 1)
               continue;
            LoescheBetriebdbEntry(betrieb);
         }
      }
   }

   // Aufruf der Funktion von GP-Rolle
   else
   {
      BSTCont<AnsArbBereich> ansarb;
      GetArbeitsbereichListe(ansarb,int_null(),L"",betnr);
      if(ansarb.Count() >= 1)
         return;

      LoescheBetriebdbEntry(betnr);
   }
}

void MgrBsa::LoescheBetriebdbEntry(const CString& betnr)
{
   BSASSERT(!betnr.IsEmpty());

   BSPtr<ZDDataStruct> pZD  (new BETRIEBDB);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   pB->SetString  (BETRIEBDB::BETNR,  betnr);
   pB->SetLongNull(BETRIEBDB::VPDID_, GetVPDID());

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
         return;
      else
         throw;
   }
   pB->Delete();
}

long_null MgrBsa::GetVPDID()
{
   BSPtr<ZDMultiAccess> pAcc(new BETRIEBDB::ReadVPDID());
   BSPtr<BZBoBase> pBo(new BZBoBase(new BETRIEBDB));

   BSASSERT(pAcc.get());
   try
   {
      pBo->Read(pAcc);
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
         return long_null();
      else
         throw;
   }
   return pBo->GetLongNull(BETRIEBDB::VPDID_);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetDefaultPrinter(const REQDrucker& oREQ, ANSDrucker& oANS)
{
   BSPtr<BZBoBase>     pB   (new BoDruckerZuordnung());
   BSPtr<ZDListAccess> pList(new DRUCKERZUORDNUNG::GetDefaultPrinter);
   BZBoTList<BoDruckerZuordnung> bolist(pB, pList);
   bolist.Param()->SetString( DRUCKERZUORDNUNG::GetDefaultPrinter::Param::DOKUMENT_EQ , oREQ.dokument );
   bolist.Param()->SetIntNull(DRUCKERZUORDNUNG::GetDefaultPrinter::Param::MANDANTNR_EQ, oREQ.mandantnr);

   bolist.Read();

   bool windowsdruckererf = false;
   CString relevanterkontext;
   bool foundDok = SetAndCheckDokument(oREQ.dokument, windowsdruckererf, relevanterkontext);

   // Arbeitsplatz und Lokationlfdnr herausfinden
   CString arbeitsplatz = oREQ.arbeitsplatz;
   long lokationlfdnr = -1;
   SetAndCheckArbeitsplatz(arbeitsplatz, lokationlfdnr);

   // Default Initialisierung
   REQCheckDefaultPrinter oReqCheck;
   oReqCheck.checkKontext          = true;
   oReqCheck.checkDokumentart      = true;
   oReqCheck.checkArbeitsplatz     = true;
   oReqCheck.checkLokation         = false;

   BoDruckerZuordnung* pBo;

   int paranoia(0);
   while(true)
   {
      for(int i = 0; i < bolist.Count(); i++)
      {
         pBo = dynamic_cast<BoDruckerZuordnung*>(bolist.GetBO(i));

         if(!CheckPrinterForDefault(pBo, oREQ, relevanterkontext, arbeitsplatz, lokationlfdnr, oReqCheck))
            continue;

         REQDruckDok oREQDruckDok;
         oREQDruckDok.dokument = oREQ.dokument;
         oREQDruckDok.drucker = pBo->GetString(DRUCKERZUORDNUNG::DRUCKERBEZ);

         if(!CheckDruckerDokument(&oREQDruckDok, windowsdruckererf, foundDok))
            return;

         // Wenn bis hier her gekommen, dann wurde passender Eintrag gefunden
         oANS.drucker          = pBo->Get(DRUCKERZUORDNUNG::DRUCKERBEZ      );
         oANS.druckerformlfdnr = pBo->Get(DRUCKERZUORDNUNG::DRUCKERFORMLFDNR);
         oANS.kassakz          = pBo->Get(DRUCKERZUORDNUNG::KASSAKZ         );
         return;
      }

      // Herausfinden des passendsten Druckers:
      // Vorgegangen wird nach folgendem Entscheidungsmuster:
      // Arbeitsplatz   Lokation   Kontext  Dokumentart
      //     X             -          X          X
      //     X             -          -          X
      //     X             -          X          -
      //     X             -          -          -
      //     -             X          X          X
      //     -             X          -          X
      //     -             X          X          -
      //     -             X          -          -
      //     -             -          X          X
      //     -             -          -          X
      //     -             -          X          -
      //     -             -          -          -
      if(oReqCheck.checkKontext)
         oReqCheck.checkKontext = false;
      else
      {
         oReqCheck.checkKontext = true;
         if(oReqCheck.checkDokumentart)
            oReqCheck.checkDokumentart = false;
         else
         {
            oReqCheck.checkDokumentart = true;
            if(oReqCheck.checkArbeitsplatz)
            {
               oReqCheck.checkArbeitsplatz = false;
               oReqCheck.checkLokation     = true;
            }
            else
            {
               if(oReqCheck.checkLokation)
                  oReqCheck.checkLokation = false;
               else
                  return;    // keinen default-Drucker gefunden
            }
         }
      }

      paranoia++;
      if(paranoia > 1000)
         break;
   }
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::CheckPrinterForDefault(BoDruckerZuordnung* pBo, const REQDrucker& oReqDrucker,
                                    const CString& relevanterkontext,
                                    const CString& arbeitsplatz, 
                                    const long& lokationlfdnr,
                                    const REQCheckDefaultPrinter& oReqCheck)
{
   // Wenn Kassakz erforderlich und der Drucker hat keines, dann
   // diese Zuordnung bergehen
   if(oReqDrucker.nurkassa.Get(false) && pBo->IsNull(DRUCKERZUORDNUNG::KASSAKZ))
      return false;

   if( oReqCheck.checkKontext )
   {
      if(!CheckKontext(*pBo, oReqDrucker, relevanterkontext))
         return false;
   }
   else
   {
      // Wenn Kontext nicht mehr zu prfen ist, aber im Model Druckerzuordnung
      // ein Kontext enthalten ist, so ist dieser Eintrag auch nicht
      // zutreffend
      if(!pBo->IsNull(DRUCKERZUORDNUNG::VERRFABRIKATKZ  )            ||
          pBo->GetBoolNull(DRUCKERZUORDNUNG::WELTAUTOKZ ).Get(false) ||
         !pBo->IsNull(DRUCKERZUORDNUNG::MANDANTNR       )            ||
         !pBo->IsNull(DRUCKERZUORDNUNG::MODULNR         )            ||
         !pBo->IsNull(DRUCKERZUORDNUNG::NUR_MANDANTNR   )            ||
         !pBo->IsNull(DRUCKERZUORDNUNG::VERTRIEBSPARTNER)              )
         return false;
   }

   if( oReqCheck.checkDokumentart )
   {
      if( pBo->GetString(DRUCKERZUORDNUNG::DOKUMENT) != oReqDrucker.dokument )
         return false;
   }
   else
   {
      if( !pBo->IsNull(DRUCKERZUORDNUNG::DOKUMENT) )
         return false;
   }

   if( oReqCheck.checkArbeitsplatz )
   {
      if( pBo->GetString(DRUCKERZUORDNUNG::ARBEITSPLATZBEZ) != arbeitsplatz)
         return false;
   }
   else
   {
      if(!pBo->IsNull(DRUCKERZUORDNUNG::ARBEITSPLATZBEZ))
         return false;
   }

   if( oReqCheck.checkLokation && lokationlfdnr > 0)
   {
      if( pBo->GetLongNull(DRUCKERZUORDNUNG::LOKATIONLFDNR).Get(0) != lokationlfdnr)
         return false;
   }
   else
   {
      if(!pBo->IsNull(DRUCKERZUORDNUNG::LOKATIONLFDNR))
         return false;
   }

   return true;
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::SetAndCheckDokument(const CString& dokument, 
                                 bool& windowsdruckererf, CString& relevanterkontext)
{
   if(dokument.IsEmpty())
      return true;

   BZBoBase oBoDok(new DOKUMENT());
   oBoDok.SetString(DOKUMENT::DOKUMENTART, dokument);

   try
   {
      oBoDok.Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         return false;
      }
      else
         throw;
   }

   windowsdruckererf = oBoDok.GetBoolNull(DOKUMENT::WINDOWSDRUCKERERF).Get(false);
   relevanterkontext = oBoDok.GetString(DOKUMENT::RELEVANTERKONTEXT);

   return true;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::SetAndCheckArbeitsplatz(CString& arbeitsplatz, long& lokationlfdnr)
{
   if(arbeitsplatz.IsEmpty() && ZDTsGlobal::ArbeitsplatzDefined())
      arbeitsplatz = ZDTsGlobal::GetArbeitsplatz();

   if(arbeitsplatz.IsEmpty())
      return;

   IMgrInif::ANSArbeitsplatz oANSAplatz;

   try
   {
      IMgrInif::Create()->GetArbeitsplatz(arbeitsplatz, oANSAplatz);
   }
   catch(const BZException& e) 
   {
      if(e.GetId() != InifId::ARBEITSPLATZ_NOTFOUND)
         throw;
   }

   lokationlfdnr = oANSAplatz.lokationlfdnr.Get(-1);
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::CheckDruckerDokument(REQDruckDok* pREQ, bool windowsdruckererf, bool foundDok)
{
   BSPtr<ZDDataStruct> pZD  (new DRUCKER);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new DRUCKER::Liste2);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(DRUCKER::Liste2::Param::DRUCKERBEZ_EQ, pREQ->drucker);
   bolist.Read();

   if(bolist.Count() <= 0 || foundDok == false)
      return false;

   if(bolist.GetBO(0)->GetBoolNull(DRUCKER::ISWINDOWSDRUCKER).Get(false) != windowsdruckererf)
      BSTHROW BZException(i18n("Gewhlter Drucker ist fr das gewhlte Dokument nicht gltig (lokal/Server)\n z.B.: Das Dokument ist nur fr Server-Drucker gltig, der gewhlte Drucker ist aber lokal definiert."),
         BsaId::DRUCKERZUORD_NOTFOUND);

   return true;
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::CheckKontext(const BoDruckerZuordnung& bo, const REQDrucker& oREQ, CString relevanterkontext)
{
   BoDruckerZuordnung::KONTEXT kontext = bo.WhichKontextIsRelevant(relevanterkontext);

   if(kontext == BoDruckerZuordnung::VERRFABRIKAT)
   {
      if(bo.GetString(DRUCKERZUORDNUNG::VERRFABRIKATKZ) == oREQ.verrfabrikatkz)
         return true;
      else
         return false;
   }

   if(kontext == BoDruckerZuordnung::WELTAUTO)
   {
      if(bo.GetBoolNull(DRUCKERZUORDNUNG::WELTAUTOKZ).Get(false) == oREQ.weltautokz.Get(false))
         return true;
      else
         return false;
   }

   if(kontext == BoDruckerZuordnung::ARBEITSBEREICH)
   {
      if(bo.GetIntNull(DRUCKERZUORDNUNG::MANDANTNR).Get(0) == oREQ.mModul.mandantnr.Get(-1) &&
         bo.GetIntNull(DRUCKERZUORDNUNG::MODULNR  ).Get(0) == oREQ.mModul.modulnr.Get(-1)     )
         return true;
      else
         return false;
   }

   if(kontext == BoDruckerZuordnung::NUR_MANDANTNR)
   {
      if(bo.GetIntNull(DRUCKERZUORDNUNG::NUR_MANDANTNR).Get(0) == oREQ.mModul.mandantnr.Get(-1))
         return true;
      else
         return false;
   }

   if(kontext == BoDruckerZuordnung::VERRFAB_VP)
   {
      if(   bo.Get(DRUCKERZUORDNUNG::VERTRIEBSPARTNER).Get(0) == oREQ.vertriebspartner.Get(-1)
         && bo.Get(DRUCKERZUORDNUNG::VERRFABRIKATKZ  )        == oREQ.verrfabrikatkz          )
         return true;
      else
      {
         if(   bo.Get(DRUCKERZUORDNUNG::VERTRIEBSPARTNER).Get(0) == oREQ.vertriebspartner.Get(-1)
            && bo.IsNull(DRUCKERZUORDNUNG::VERRFABRIKATKZ))
            return true;
         else
         {
            if(   bo.Get(DRUCKERZUORDNUNG::VERRFABRIKATKZ) == oREQ.verrfabrikatkz
               && bo.IsNull(DRUCKERZUORDNUNG::VERTRIEBSPARTNER))
               return true;
            else
               return false;
         }
         return false;
      }
   }

   // Wenn im Model irgendein Kontext steht, der nicht zutrifft, dann return false
   if(!bo.IsNull(DRUCKERZUORDNUNG::VERRFABRIKATKZ  )            ||
       bo.GetBoolNull(DRUCKERZUORDNUNG::WELTAUTOKZ ).Get(false) ||
      !bo.IsNull(DRUCKERZUORDNUNG::MANDANTNR       )            ||
      !bo.IsNull(DRUCKERZUORDNUNG::MODULNR         )            ||
      !bo.IsNull(DRUCKERZUORDNUNG::NUR_MANDANTNR   )            ||
      !bo.IsNull(DRUCKERZUORDNUNG::VERTRIEBSPARTNER)              )
      return false;

   return true;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetPrinterStort(const REQPrinterStort& oREQ, ANSDrucker& oANS)
{
   BSASSERT(!oREQ.oMM.mandantnr.IsNull() && !oREQ.oMM.modulnr.IsNull());
   BSASSERT(!oREQ.standort.IsEmpty());
   BSASSERT(!oREQ.dokument.IsEmpty());

   BZBoBase oBO(new DRUCKERZUORDSTORT);
   BSPtr<ZDMultiAccess> pAcc(new DRUCKERZUORDSTORT::DruckerZuordStort());

   oBO.SetIntNull(DRUCKERZUORDSTORT::MANDANTNR, oREQ.oMM.mandantnr);
   oBO.SetIntNull(DRUCKERZUORDSTORT::MODULNR  , oREQ.oMM.modulnr  );
   oBO.SetString( DRUCKERZUORDSTORT::STANDORT , oREQ.standort     );
   oBO.SetString( DRUCKERZUORDSTORT::DOKUMENT , oREQ.dokument     );

   try
   {
      oBO.Read(pAcc);
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Druckerzuordnung fr Tagesende (standortabhngig) mit MandantNr: %1, "\
            "ModulNr: %2, Standort: %3 und Dokument: %4 nicht gefunden."), BsaId::DRUCKERZUORDSTORT_NOTFOUND)
            .Arg(oREQ.oMM.mandantnr.Get()).Arg(oREQ.oMM.modulnr.Get()).Arg(oREQ.standort).Arg(oREQ.dokument);
      }
      throw;
   }

   oANS.drucker = oBO.GetString(DRUCKERZUORDSTORT::DRUCKERBEZ);
   oANS.kassakz = oBO.GetString(DRUCKERZUORDSTORT::KASSAKZ);
   oANS.druckerformlfdnr = oBO.GetLongNull(DRUCKERZUORDSTORT::DRUCKERFORMLFDNR);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::AddTagendProg(REQTagendProg& req)
{
 //  BSASSERT(req.tagendprogart != L"");

   // Prfen ob bereits ein Satz mit den gleichen Parametern vorhanden ist
   BSPtr<ZDDataStruct> pZD  (new TAGENDPROG);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new TAGENDPROG::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(TAGENDPROG::Liste::Param::TAGENDPROGART_EQ, req.tagendprogart);
   bolist.Param()->SetString(TAGENDPROG::Liste::Param::PARAMETER_EQ, req.parameter);

   bolist.Read();

   // Wenn der gleiche Satz bereits vorhanden ist -> OK
   if(bolist.Count() > 0)
      return;

   // Einfgen des gewnschten Satzes
   BSPtr<ZDDataStruct> pZDNew  (new TAGENDPROG);
   BSPtr<BZBoBase>     pBNew  (new BZBoBase(pZDNew));

   pBNew->SetState (BZBoBase::ISNEW);
   pBNew->SetString(TAGENDPROG::TAGENDPROGART   , req.tagendprogart);
   pBNew->SetString(TAGENDPROG::PARAMETER       , req.parameter);

   pBNew->Persist();

   return;
   
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetTagendProgartListe(const CString& tagendprogart, BSTCont<ANSTagendProg>& ans)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD  (new TAGENDPROG);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new TAGENDPROG::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(TAGENDPROG::Liste::Param::TAGENDPROGART_EQ, tagendprogart);
   bolist.Read();

   for(int iter=0; iter<bolist.Count(); iter++)
   {
      BZBoBase* pBo = bolist.GetBO(iter);

      BSPtr<ANSTagendProg> pItem(new ANSTagendProg());
      pItem->tagendproglfdnr = pBo->GetLongNull(TAGENDPROG::TAGENDPROGLFDNR).Get();
      pItem->tagendprogart   = pBo->GetString  (TAGENDPROG::TAGENDPROGART);
      pItem->parameter       = pBo->GetString  (TAGENDPROG::PARAMETER);
      ans.Insert(pItem.release());
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetNgeGFFromNgeGF(const REQNgeGFFromNgeGF& req, AnsIntGFNGE& ans)
{
   if(req.fromMModul.mandantnr.Get(-1) < 0 || req.fromMModul.modulnr.Get(-1) < 0 || 
      req.toMModul.mandantnr.Get(-1) < 0   || req.toMModul.modulnr.Get(-1) < 0     )
      BSTHROW BZException(i18n("Arbeitsbereich nicht gltig"), BsaId::NGEGF2NGEGF_NOTFOUND);

   if(req.fromMModul.mandantnr != req.toMModul.mandantnr)
      BSTHROW BZException(i18n("MandantNr. darf nicht abweichen"), BsaId::NGEGF2NGEGF_NOTFOUND);

   if(req.fromGfLfdnr.Get(-1) < 0)
      // HINT: Laufende Nummer fr internen Geschftsfall
      BSTHROW BZException(i18n("LfdNr. fr int. GF nicht gltig"), BsaId::NGEGF2NGEGF_NOTFOUND);

   bool foundInivalueInRange;
   CString dummy = csIni->GetNeutralKey(req.toZweck, L"INTGFZWECK", &foundInivalueInRange);
   if(!foundInivalueInRange)
      BSTHROW BZException(i18n("Zweck nicht gltig"), BsaId::NGEGF2NGEGF_NOTFOUND);

   // berprfen ob GF-Zuordnung eindeutig
   BSPtr<BZBoBase>     pBCheck   (new BZBoBase(new NGEGF2NGEGF));
   BSPtr<ZDListAccess> pListCheck(new NGEGF2NGEGF::Liste1Check);
   BZBoList bolistCheck(pBCheck, pListCheck);

   //Check-Parameter setzen
   bolistCheck.Param()->SetIntNull( NGEGF2NGEGF::Liste1Check::Param::MANDANTNR_EQ,  req.fromMModul.mandantnr);
   bolistCheck.Param()->SetIntNull( NGEGF2NGEGF::Liste1Check::Param::MODULNR1_EQ,   req.fromMModul.modulnr  );
   bolistCheck.Param()->SetLongNull(NGEGF2NGEGF::Liste1Check::Param::NGEGFLFDNR_EQ, req.fromGfLfdnr         );
   bolistCheck.Param()->SetIntNull( NGEGF2NGEGF::Liste1Check::Param::MODULNR2_EQ,   req.toMModul.modulnr    );
   bolistCheck.Param()->SetString(  NGEGF2NGEGF::Liste1Check::Param::ZWECK_EQ,      req.toZweck             );
   //Check-Liste lesen
   bolistCheck.Read();

   //Falls mehr als eine Zuordnung gefunden wurde --> BSTHROW
   if(bolistCheck.Count() > 1)
      BSTHROW BZException(i18n("Keine eindeutige GF-Zuordnung gefunden"), BsaId::NGEGF2NGEGF_NOTUNIQUE);

   BSPtr<BZBoBase>     pB   (new BZBoBase(new NGEGF2NGEGF));
   BSPtr<ZDListAccess> pList(new NGEGF2NGEGF::Liste1);
   BZBoList bolist(pB, pList);

   // Fixe Parameter
   bolist.Param()->SetIntNull( NGEGF2NGEGF::Liste1::Param::MANDANTNR_EQ,   req.fromMModul.mandantnr);
   bolist.Param()->SetString(  NGEGF2NGEGF::Liste1::Param::ZWECK_EQ,       req.toZweck             );

   // Bidirektionales lesen
   bolist.Param()->SetIntNull( NGEGF2NGEGF::Liste1::Param::MODULNR1_EQ,    req.fromMModul.modulnr);
   bolist.Param()->SetIntNull( NGEGF2NGEGF::Liste1::Param::MODULNR2_EQ,    req.toMModul.modulnr  );
   bolist.Param()->SetLongNull(NGEGF2NGEGF::Liste1::Param::NGEGFLFDNR1_EQ, req.fromGfLfdnr       );
   bolist.Read();

   if(bolist.Count() <= 0)
   {
      bolist.Param()->SetIntNull( NGEGF2NGEGF::Liste1::Param::MODULNR1_EQ,    req.toMModul.modulnr  );
      bolist.Param()->SetIntNull( NGEGF2NGEGF::Liste1::Param::MODULNR2_EQ,    req.fromMModul.modulnr);
      bolist.Param()->SetLongNull(NGEGF2NGEGF::Liste1::Param::NGEGFLFDNR2_EQ, req.fromGfLfdnr       );
      bolist.Param()->SetNull(    NGEGF2NGEGF::Liste1::Param::NGEGFLFDNR1_EQ                        );
      bolist.Read();
   }

   if(bolist.Count() <= 0)
      ans.gf_found = false;
   else
   {
      BZBoBase* pBoGf = bolist.GetBO(0);
      int lfdnr1 = pBoGf->GetLongNull(NGEGF2NGEGF::NGEGFLFDNR1).Get();
      int lfdnr2 = pBoGf->GetLongNull(NGEGF2NGEGF::NGEGFLFDNR2).Get();

      ans.lfdnr               = (lfdnr1 == req.fromGfLfdnr.Get()) ? lfdnr2 : lfdnr1;
      ans.bezeichnung         = pBoGf->Get(NGEGF2NGEGF::BEZEICHNUNG);
      ans.kurzbezeichnung     = pBoGf->Get(NGEGF2NGEGF::KURZBEZ);
      ans.verbuchung          = pBoGf->Get(NGEGF2NGEGF::VERBUCHUNG);
      ans.iskalkulatorisch    = pBoGf->Get(NGEGF2NGEGF::ISKALKULATORISCH).Get();
      ans.isgwreparatur       = pBoGf->Get(NGEGF2NGEGF::ISGWREPARATUR   ).Get();
      ans.isekbeeinflussend   = pBoGf->Get(NGEGF2NGEGF::ISEKBEEINFLUSSEND).Get();
      ans.zweck               = pBoGf->Get(NGEGF2NGEGF::ZWECK);
      ans.ergebniswirksam     = pBoGf->Get(NGEGF2NGEGF::ERGEBNISWIRKSAM);
      ans.wagentyp            = pBoGf->Get(NGEGF2NGEGF::WAGENTYP);
      ans.einkverkkz          = pBoGf->Get(NGEGF2NGEGF::EINKVERKKZ);
      ans.leistungsart        = pBoGf->Get(NGEGF2NGEGF::LEISTUNGSART);
      ans.stueckwirksam       = pBoGf->Get(NGEGF2NGEGF::STUECKWIRKSAM).Get();
      ans.verbuchen           = pBoGf->Get(NGEGF2NGEGF::VERBUCHEN).Get();
      ans.absatzkanalrechempf = pBoGf->Get(NGEGF2NGEGF::ABSATZKANALRECHEMPF).Get();
      ans.gf_found            = true;
   }
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::ExistsNgeGFFromNgeGF(const MModul &oMModul, long_null gfLfdNr, CString IntGFZweck)
{
   if(gfLfdNr.Get(-1) < 0)
      // HINT: Laufende Nummer fr internen Geschftsfall
      BSTHROW BZException(i18n("LfdNr. fr int. GF nicht gltig"), BsaId::NGEGF2NGEGF_NOTFOUND);

   // berprfen ob GF-Zuordnung eindeutig
   BSPtr<BZBoBase>     pBCheck   (new BZBoBase(new NGEGF2NGEGF));
   BSPtr<ZDListAccess> pListCheck(new NGEGF2NGEGF::Liste1Check);
   BZBoList bolistCheck(pBCheck, pListCheck);

   //Check-Parameter setzen
   bolistCheck.Param()->SetIntNull( NGEGF2NGEGF::Liste1Check::Param::MANDANTNR_EQ,  oMModul.mandantnr);
   bolistCheck.Param()->SetIntNull( NGEGF2NGEGF::Liste1Check::Param::MODULNR1_EQ,   oMModul.modulnr);
   bolistCheck.Param()->SetLongNull(NGEGF2NGEGF::Liste1Check::Param::NGEGFLFDNR_EQ, gfLfdNr);
   bolistCheck.Param()->SetString(  NGEGF2NGEGF::Liste1Check::Param::ZWECK_EQ,      IntGFZweck);
   //Check-Liste lesen
   bolistCheck.Read();

   //Falls mehr als eine Zuordnung gefunden wurde --> return true
   if(bolistCheck.Count() >= 1)
      return true;

   return false;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetMechGruppe(REQMechGruppe& oREQ, 
                           ANSMechGruppe& oANS, 
                           bool incl_deleted)
{
   BSPtr<ZDDataStruct> pZD(new MECHGRUPPE);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetInt(MECHGRUPPE::MANDANTNR,  oREQ.mandantnr.Get());
   pB->SetInt(MECHGRUPPE::MODULNR,    oREQ.modulnr.Get());
   pB->SetLong(MECHGRUPPE::GRUPPENNR, oREQ.gruppennr.Get());

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Mechgruppe mit Mandantnr: %1, Modulnr: %2 und Gruppennr %3 nicht gefunden."),
                              BsaId::MECHGRUPPE_NOTFOUND).Arg(oREQ.mandantnr).Arg(oREQ.modulnr).Arg(oREQ.gruppennr);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(MECHGRUPPE::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Mechgruppe mit Mandantnr: %1, Modulnr: %2 und Gruppennr %3 ist als gelscht gekennzeichnet."),
                           BsaId::MECHGRUPPE_NOTFOUND).Arg(oREQ.mandantnr).Arg(oREQ.modulnr).Arg(oREQ.gruppennr);
   }

   oANS.gruppenbez = pB->GetString(MECHGRUPPE::GRUPPENBEZ);
   oANS.gruppennr  = pB->GetLongNull(MECHGRUPPE::GRUPPENNR);
   oANS.gptnrkdb   = pB->GetLongNull(MECHGRUPPE::GPTNRKDB);
   oANS.loeschkz   = pB->GetBoolNull(MECHGRUPPE::LOESCHKZ).Get(false);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetListeMechGruppe(const MModul& req, BSTCont<ANSMechGruppe>& ans, bool incl_deleted)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD  (new MECHGRUPPE);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new MECHGRUPPE::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetInt(MECHGRUPPE::Liste::Param::MANDANTNR_EQ, req.mandantnr.Get(-1));
   bolist.Param()->SetInt(MECHGRUPPE::Liste::Param::MODULNR_EQ,   req.modulnr.Get(-1));
   bolist.Read();

   for(int iter=0; iter<bolist.Count(); iter++)
   {
      BZBoBase* pBoMech = bolist.GetBO(iter);

      // Als gelscht markierte Eintrge auf Wunsch nicht liefern
      if(incl_deleted == false && pBoMech->GetBoolNull(MECHGRUPPE::LOESCHKZ).Get(false) == true)
         continue;

      BSPtr<ANSMechGruppe> pItem(new ANSMechGruppe());
      pItem->gruppenbez = pBoMech->GetString(MECHGRUPPE::GRUPPENBEZ);
      pItem->gruppennr  = pBoMech->GetLongNull(MECHGRUPPE::GRUPPENNR);
      pItem->gptnrkdb   = pBoMech->GetLongNull(MECHGRUPPE::GPTNRKDB);
      pItem->loeschkz   = pBoMech->GetBoolNull(MECHGRUPPE::LOESCHKZ).Get(false);
      ans.Insert(pItem.release());
   }
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetDruckerFormular(const long druckformlfdnr, ANSDruckerForm& ans)
{
   BZBoList bolist(new BZBoBase(new DRUCKERFORMULAR), new DRUCKERFORMULAR::GetAllDruckDaten);
   bolist.Param()->SetLong(DRUCKERFORMULAR::GetAllDruckDaten::Param::DRUCKERFORMLFDNR_EQ, druckformlfdnr);
   bolist.Read();

   if(bolist.Count() <= 0)
      BSTHROW BZException(i18n("Druckerformular: Satz mit Lfd.Nr. %1 nicht gefunden"), BsaId::DRUCKER_NOTFOUND)
         .Arg(druckformlfdnr);

   BZBoBase* pBo = bolist.GetBO(0);

   ans.druckerbez                = pBo->Get(DRUCKERFORMULAR::DRUCKERBEZ               );
   ans.bezeichnung               = pBo->Get(DRUCKERFORMULAR::BEZEICHNUNG              );
   ans.formularlfdnr             = pBo->Get(DRUCKERFORMULAR::FORMULARLFDNR            );
   ans.formularbez               = pBo->Get(DRUCKERFORMULAR::FORMULARBEZ              );
   ans.betriebsname              = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME             );
   ans.betriebsname_elektr       = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_ELEKTR      );
   ans.briefkopf_absender        = pBo->Get(DRUCKERFORMULAR::BRIEFKOPF_ABSENDER       );
   ans.betriebsnr_fix            = pBo->Get(DRUCKERFORMULAR::BETRIEBSNR_FIX           );
   ans.footer_spalte1            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE1           );
   ans.footer_spalte2            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE2           );
   ans.footer_spalte3            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE3           );
   ans.footer_spalte4            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE4           );
   ans.footer_spalte5            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE5           );
   ans.footer_spalte6            = pBo->Get(DRUCKERFORMULAR::FOOTER_SPALTE6           );
   ans.footer_image_lfdnr1       = pBo->Get(DRUCKERFORMULAR::FOOTER_IMAGE1            );
   ans.footer_image_lfdnr2       = pBo->Get(DRUCKERFORMULAR::FOOTER_IMAGE2            );
   ans.footer_image_lfdnr3       = pBo->Get(DRUCKERFORMULAR::FOOTER_IMAGE3            );
   ans.footer_image1_bez         = pBo->Get(DRUCKERFORMULAR::IMAGE1                   );
   ans.footer_image2_bez         = pBo->Get(DRUCKERFORMULAR::IMAGE2                   );
   ans.footer_image3_bez         = pBo->Get(DRUCKERFORMULAR::IMAGE3                   );
   ans.footer_image1_filename    = pBo->Get(DRUCKERFORMULAR::FILE1                    );
   ans.footer_image2_filename    = pBo->Get(DRUCKERFORMULAR::FILE2                    );
   ans.footer_image3_filename    = pBo->Get(DRUCKERFORMULAR::FILE3                    );
   ans.markenlogo_andruck        = pBo->Get(DRUCKERFORMULAR::MARKENLOGO_ANDRUCK       );
   ans.markenlogo_andruck_elektr = pBo->Get(DRUCKERFORMULAR::MARKENLOGO_ANDRUCK_ELEKTR);
   ans.mehrmarken                = pBo->Get(DRUCKERFORMULAR::MEHRMARKEN               );
   ans.markenauftrittlfdnr       = pBo->Get(DRUCKERFORMULAR::MARKENAUFTRITTLFDNR      );
   ans.markenauftritt_bez        = pBo->Get(DRUCKERFORMULAR::MARKEN_BEZ               );
   ans.markenauftritt_font       = pBo->Get(DRUCKERFORMULAR::MARKENFONT               );

   FillMarkenlogos(ans, pBo);

   ans.topmargin                          = pBo->Get(DRUCKERFORMULAR::TOPMARGIN                    );
   ans.bottommargin                       = pBo->Get(DRUCKERFORMULAR::BOTTOMMARGIN                 );
   ans.footer_pia                         = pBo->Get(DRUCKERFORMULAR::FOOTER_PIA                   );
   ans.footer_pia_aktiv                   = pBo->Get(DRUCKERFORMULAR::FOOTER_PIA_AKTIV             );
   ans.betriebsname_aktiv                 = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_AKTIV           );
   ans.betriebsname_aktiv_elektr          = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_AKTIV_ELEKTR    );
   ans.betriebsname_zusatz1               = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_ZUS1            );
   ans.betriebsname_zusatz1_elektr        = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_ZUS1_ELEKTR     );
   ans.betriebsname_zusatz2               = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_ZUS2            );
   ans.betriebsname_zusatz2_elektr        = pBo->Get(DRUCKERFORMULAR::BETRIEBSNAME_ZUS2_ELEKTR     );
   ans.betriebslogo_image_lfdnr           = pBo->Get(DRUCKERFORMULAR::BETRIEBSLOGO_IMAGE           );
   ans.betriebslogo_image_lfdnr_elektr    = pBo->Get(DRUCKERFORMULAR::BETRIEBSLOGO_IMAGE_ELEKTR    );
   ans.betriebslogo_image_bez             = pBo->Get(DRUCKERFORMULAR::IMAGE4                       );
   ans.betriebslogo_image_bez_elektr      = pBo->Get(DRUCKERFORMULAR::IMAGE6                       );
   ans.betriebslogo_image_filename        = pBo->Get(DRUCKERFORMULAR::FILE4                        );
   ans.betriebslogo_image_filename_elektr = pBo->Get(DRUCKERFORMULAR::FILE6                        );
   ans.footerabweichung                   = pBo->Get(DRUCKERFORMULAR::FOOTERABWEICHUNG             );
   ans.markenlogo_ausrichtung             = pBo->Get(DRUCKERFORMULAR::MARKENLOGO_AUSRICHTUNG       );
   ans.markenlogo_ausrichtung_elektr      = pBo->Get(DRUCKERFORMULAR::MARKENLOGO_AUSRICHTUNG_ELEKTR);
   ans.header_betriebsdaten               = pBo->Get(DRUCKERFORMULAR::HEADER_BETRIEBSDATEN         );
   ans.nur_markenfont_verwenden           = pBo->Get(DRUCKERFORMULAR::NUR_MARKENFONT_VERWENDEN     );

   // Wenn Formularimage::ISSTANDARDIMAGE == false UND nicht absoluter Pfad
   // fr Formularimage::FILENAME eingegeben, dann mit $KFZDEALERTOPDIR ergnzen
   if(!ans.footer_image1_filename.IsEmpty()      && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE1).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.footer_image1_filename);
   if(!ans.footer_image2_filename.IsEmpty()      && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE2).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.footer_image2_filename);
   if(!ans.footer_image3_filename.IsEmpty()      && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE3).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.footer_image3_filename);
   if(!ans.betriebslogo_image_filename.IsEmpty() && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE4).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.betriebslogo_image_filename);
   if(!ans.betriebslogo_image_filename_elektr.IsEmpty() && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE6).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.betriebslogo_image_filename_elektr);

   GetImageSize(GetImagePath(ans.footer_image1_filename), ans.footer_image1_width, ans.footer_image1_height);
   GetImageSize(GetImagePath(ans.footer_image2_filename), ans.footer_image2_width, ans.footer_image2_height);
   GetImageSize(GetImagePath(ans.footer_image3_filename), ans.footer_image3_width, ans.footer_image3_height);

   GetImageSize(GetImagePath(ans.betriebslogo_image_filename       ), ans.betriebslogo_image_width       , ans.betriebslogo_image_height       );
   GetImageSize(GetImagePath(ans.betriebslogo_image_filename_elektr), ans.betriebslogo_image_width_elektr, ans.betriebslogo_image_height_elektr);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::AddKFZDEALERTOPDIRIfIsRelative(CString & filename)
{
   if (BSDir().IsRelative(filename))
   {
      CString kfzTopDir = ZDTsGlobal::GetEnv(L"KFZDEALERTOPDIR");
      if (kfzTopDir.IsEmpty())
         BSTHROW BSException (i18n("Env-Variable KFZDEALERTOPDIR nicht gesetzt."));
      filename = kfzTopDir + BSDir().Separator() + filename;
   }
}
////////////////////////////////////////////////////////////////////////////////
int MgrBsa::GetDefaultTopMargin() const
{
   return DEFAULT_TOPMARGIN;
}

////////////////////////////////////////////////////////////////////////////////
int MgrBsa::GetDefaultBottomMargin() const
{
   return DEFAULT_BOTTOMMARGIN;
}

////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetImagePath( const CString& strPath )
{
   if ( strPath.IsEmpty() || !BSDir::IsRelative( strPath ) )
      return strPath;

   CString strResult = ZDTsGlobal::GetEnv( L"KFZVERSTOPDIR" );

   if ( strResult.IsEmpty() )
   {
      BSLog( BS_INFO, i18n("Env-Variable KFZVERSTOPDIR nicht gesetzt.") );
      return strPath;
   }

   strResult += BSDir::Separator();
   strResult += strPath;

   return strResult;
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetFabBetrieb(const CString& betrieb, const CString& fabrikat,
                           CString& fabbetrieb)
{
   BSPtr<ZDDataStruct> pZD(new BETNRZUORD);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetString(BETNRZUORD::BETNR,      betrieb);
   pB->SetString(BETNRZUORD::FABRIKATKZ, fabrikat);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Eintrag in Betnrzuord nicht gefunden: Betrieb %1 Fabrikat %2"),
                                 BsaId::BETNRZUORD_NOTFOUND).Arg(betrieb).Arg(fabrikat);
      }
      else
         throw;
   }
   fabbetrieb = pB->GetString(BETNRZUORD::FABBETNR);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetAbwRVSnr(const CString& betrieb, const CString& fabrikat,
                         CString& abwrvsnr)
{
   BSPtr<ZDDataStruct> pZD(new BETNRZUORD);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetString(BETNRZUORD::BETNR,      betrieb);
   pB->SetString(BETNRZUORD::FABRIKATKZ, fabrikat);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Eintrag in Betnrzuord nicht gefunden: Betrieb %1 Fabrikat %2"),
                                 BsaId::BETNRZUORD_NOTFOUND).Arg(betrieb).Arg(fabrikat);
      }
      else
         throw;
   }
   abwrvsnr = pB->GetString(BETNRZUORD::ABWRVSNR);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetWerkBetrieb(const CString& betrieb, const CString& fabrikat,
                            CString& werkbetrieb)
{
   BSPtr<ZDDataStruct> pZD(new BETNRZUORD);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetString(BETNRZUORD::BETNR,      betrieb);
   pB->SetString(BETNRZUORD::FABRIKATKZ, fabrikat);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Eintrag in Betnrzuord nicht gefunden: Betrieb '%1' Fabrikat '%2'"),
                                 BsaId::BETNRZUORD_NOTFOUND).Arg(betrieb).Arg(fabrikat);
      }
      else
         throw;
   }
   werkbetrieb = pB->GetString(BETNRZUORD::WERKSBETRIEBSNR);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetFabBetriebListe(const CString& betrieb, BSTCont<ANSBetnrZuord>& oANS)
{
   oANS.Clear();

   BSPtr<ZDDataStruct> tmp(new BETNRZUORD());
   BSPtr<BZBoBase    > tmpBO(new BZBoBase(tmp));
   BSPtr<ZDListAccess> tmplist(new BETNRZUORD::FabBetriebe());
   BZBoList bolist(tmpBO, tmplist);
   bolist.Param()->SetString(BETNRZUORD::FabBetriebe::Param::BETNR_EQ,  betrieb);
   bolist.Read();

   for (int i = 0; i < bolist.Count(); i++)
   {
      BSPtr<ANSBetnrZuord> pANSBetnr(new ANSBetnrZuord());
      BZBoBase* pBo = dynamic_cast<BZBoBase*>(bolist.GetBO(i));
      BSASSERT(pBo);
      pANSBetnr->fabbetnr   = pBo->GetString(BETNRZUORD::FABBETNR);
      pANSBetnr->fabrikatkz = pBo->GetString(BETNRZUORD::FABRIKATKZ);
      pANSBetnr->betnr      = pBo->GetString(BETNRZUORD::BETNR);
      oANS.Insert(pANSBetnr.release());
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetBetriebListe(const CString& fabbetriebnr, 
                             const CString& werkbetriebnr,
                             BSTCont<ANSBetnrZuord>& oANS)
{
   oANS.Clear();

   BSPtr<ZDDataStruct> tmp(new BETNRZUORD());
   BSPtr<BZBoBase    > tmpBO(new BZBoBase(tmp));
   BSPtr<ZDListAccess> tmplist(new BETNRZUORD::Betriebe());
   BZBoList bolist(tmpBO, tmplist);
   bolist.Param()->SetString(BETNRZUORD::Betriebe::Param::FABBETNR_EQ, fabbetriebnr);
   bolist.Param()->SetString(BETNRZUORD::Betriebe::Param::WERKBETNR_EQ, werkbetriebnr);
   bolist.Read();

   for (int i = 0; i < bolist.Count(); i++)
   {
      BSPtr<ANSBetnrZuord> pANSBetnr(new ANSBetnrZuord());
      BZBoBase* pBo = dynamic_cast<BZBoBase*>(bolist.GetBO(i));
      BSASSERT(pBo);
      pANSBetnr->fabrikatkz = pBo->GetString(BETNRZUORD::FABRIKATKZ);
      pANSBetnr->betnr      = pBo->GetString(BETNRZUORD::BETNR);
      oANS.Insert(pANSBetnr.release());
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetKostenstelle(REQKostenstelle& oREQ, 
                             ANSKostenstelle& oANS, 
                             bool incl_deleted)
{
   BSPtr<ZDDataStruct> pZD(new KOSTENSTELLE);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetInt(KOSTENSTELLE::MANDANTNR,          oREQ.mandantnr.Get());
   pB->SetInt(KOSTENSTELLE::MODULNR,            oREQ.modulnr.Get());
   pB->SetString(KOSTENSTELLE::KOSTENSTELLENKZ, oREQ.kostenstellenkz);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Kostenstelle mit Mandantnr: %1, Modulnr: %2 und KostenstelleKz %3 nicht gefunden."),
               BsaId::KOSTENSTELLE_NOTFOUND).Arg(oREQ.mandantnr).Arg(oREQ.modulnr).Arg(oREQ.kostenstellenkz);
      }
      else
         throw;
   }

   if (incl_deleted == false 
       && pB->GetBoolNull(KOSTENSTELLE::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("Mechgruppe mit Mandantnr: %1, Modulnr: %2 und KostenstelleKz %3 ist als gelscht gekennzeichnet."),
                           BsaId::KOSTENSTELLE_NOTFOUND).Arg(oREQ.mandantnr).Arg(oREQ.modulnr).Arg(oREQ.kostenstellenkz);
   }

   oANS.mandantnr       = pB->GetIntNull(KOSTENSTELLE::MANDANTNR);
   oANS.modulnr         = pB->GetIntNull(KOSTENSTELLE::MODULNR);
   oANS.kostenstellenkz = pB->GetString(KOSTENSTELLE::KOSTENSTELLENKZ);
   oANS.bezeichnung     = pB->GetString(KOSTENSTELLE::BEZEICHNUNG);
   oANS.loeschkz        = pB->GetBoolNull(KOSTENSTELLE::LOESCHKZ).Get(false);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetLaserdruckeinst(REQLaserdruckeinst& oREQ, ANSLaserdruckeinst& oANS)
{
   BSPtr<ZDMultiAccess> pAcc(new LASERDRUCKEINST::ReadFuerMgrfunk());
   BSPtr<BZBoBase> pBo(new BZBoBase(new LASERDRUCKEINST));
   pBo->SetInt   (LASERDRUCKEINST::MANDANTNR, oREQ.mandantnr.Get());
   pBo->SetInt   (LASERDRUCKEINST::MODULNR,   oREQ.modulnr.Get());
   pBo->SetString(LASERDRUCKEINST::DOKUMENT,  oREQ.dokument);
   pBo->SetString(LASERDRUCKEINST::KOMBIKZ,   oREQ.kombikz);

   BSASSERT(pAcc.get());
   try
   {
      pBo->Read(pAcc);
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         bool found = false;
         CString dokart_win = csIni->GetWinText(oREQ.dokument, L"PRINTDOK", &found);
         CString kombikz_win = csIni->GetWinText(oREQ.kombikz, L"GWLKNDINTKZ", &found);

         BSTHROW BZException(i18n("Laserdruckeinstellung mit Mandantnr %1, Modulnr %2, Dokument %5 (%3) und Kombikz %6 (%4) nicht gefunden."),
               BsaId::LASERDRUCKEINST_NOTFOUND).Arg(oREQ.mandantnr).Arg(oREQ.modulnr).Arg(oREQ.dokument).Arg(oREQ.kombikz).Arg(dokart_win).Arg(kombikz_win);
      }
      else
         throw;
   }

   oANS.mandantnr                  = pBo->GetIntNull (LASERDRUCKEINST::MANDANTNR); 
   oANS.modulnr                    = pBo->GetIntNull (LASERDRUCKEINST::MODULNR);
   oANS.dokument                   = pBo->GetString  (LASERDRUCKEINST::DOKUMENT);
   oANS.kombikz                    = pBo->GetString  (LASERDRUCKEINST::KOMBIKZ);
   oANS.laserdruckeinlfdnr         = pBo->GetLongNull(LASERDRUCKEINST::LASERDRUCKEINLFDNR).Get();
   oANS.anzahlintkopien            = pBo->GetIntNull (LASERDRUCKEINST::ANZAHLINTKOPIEN).Get(0); 
   oANS.textauforiginal            = pBo->GetString  (LASERDRUCKEINST::TEXTAUFORIGINAL);
   oANS.textaufkopie1              = pBo->GetString  (LASERDRUCKEINST::TEXTAUFKOPIE1);
   oANS.textaufkopie2              = pBo->GetString  (LASERDRUCKEINST::TEXTAUFKOPIE2);
   oANS.textaufkopie3              = pBo->GetString  (LASERDRUCKEINST::TEXTAUFKOPIE3);
   oANS.textaufkopie4              = pBo->GetString  (LASERDRUCKEINST::TEXTAUFKOPIE4);
   oANS.textaufkopie5              = pBo->GetString  (LASERDRUCKEINST::TEXTAUFKOPIE5);
   oANS.duplexdruck                = pBo->GetBoolNull(LASERDRUCKEINST::DUPLEXDRUCK).Get(); // not NULL
   oANS.keinestdrueckseiteoriginal = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEORIGINAL).Get(false);
   oANS.keinestdrueckseitekopie1   = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEKOPIE1).Get(false);
   oANS.keinestdrueckseitekopie2   = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEKOPIE2).Get(false);
   oANS.keinestdrueckseitekopie3   = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEKOPIE3).Get(false);
   oANS.keinestdrueckseitekopie4   = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEKOPIE4).Get(false);
   oANS.keinestdrueckseitekopie5   = pBo->GetBoolNull(LASERDRUCKEINST::KEINESTDRUECKSEITEKOPIE5).Get(false);

   // arbeitsplatzabhngige Einstellungen lesen
   CString arbeitsplatz;
   try
   {
      arbeitsplatz = ZDTsGlobal::GetArbeitsplatz();
   }
   catch(...)
   {
      arbeitsplatz.Empty();
   }

   if (!arbeitsplatz.IsEmpty())
   {
      BSPtr<ZDMultiAccess> pAccAPSettings(new LASERDRUCKEINST::ReadFuerMgrfunk());
      BSPtr<BZBoBase> pBoAPSettings(new BZBoBase(new LASERDRUCKEINST));
      pBoAPSettings->SetInt(LASERDRUCKEINST::MANDANTNR, oREQ.mandantnr.Get());
      pBoAPSettings->SetInt(LASERDRUCKEINST::MODULNR, oREQ.modulnr.Get());
      pBoAPSettings->SetString(LASERDRUCKEINST::DOKUMENT, oREQ.dokument);
      pBoAPSettings->SetString(LASERDRUCKEINST::KOMBIKZ, oREQ.kombikz);
      pBoAPSettings->SetString(LASERDRUCKEINST::ARBEITSPLATZ, arbeitsplatz);

      if (pBoAPSettings->ReadIfExists(pAccAPSettings))
      {
         oANS.anzahlintkopien = pBoAPSettings->GetIntNull(LASERDRUCKEINST::ANZAHLINTKOPIEN).Get(0);
         oANS.textauforiginal = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFORIGINAL);
         oANS.textaufkopie1 = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFKOPIE1);
         oANS.textaufkopie2 = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFKOPIE2);
         oANS.textaufkopie3 = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFKOPIE3);
         oANS.textaufkopie4 = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFKOPIE4);
         oANS.textaufkopie5 = pBoAPSettings->GetString(LASERDRUCKEINST::TEXTAUFKOPIE5);
      }
   }
}
////////////////////////////////////////////////////////////////////////////////

int MgrBsa::GetLaserDruckKopien(const REQLaserdruckeinst& oREQ)
{
   int anzahl = 0;
                            
   IMgrBsa::REQLaserdruckeinst reqLaser;
   IMgrBsa::ANSLaserdruckeinst ansLaser;
   
   reqLaser.mandantnr = oREQ.mandantnr;
   reqLaser.modulnr   = oREQ.modulnr;
   reqLaser.dokument  = oREQ.dokument;
   reqLaser.kombikz   = oREQ.kombikz;

   try
   {
      GetLaserdruckeinst(reqLaser, ansLaser);
      anzahl = ansLaser.anzahlintkopien;            
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BsaId::LASERDRUCKEINST_NOTFOUND)
      {}
      else
         throw;
   }
            
   return anzahl;
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::DeleteTagendprogart(CString tagenendprogName, long_null tagendprogLfdnr)
{
   BSPtr<BZBoBase> pBo(new BZBoBase(new TAGENDPROG));
   pBo->SetLongNull(TAGENDPROG::TAGENDPROGLFDNR, tagendprogLfdnr);
   pBo->SetString  (TAGENDPROG::TAGENDPROGART,   tagenendprogName);

   BSPtr<ZDMultiAccess> pAccessDel(new TAGENDPROG::LoeschenAusMgrFunkt());
   BSASSERT(pAccessDel.get());
     
   pBo->Delete(pAccessDel, false, false);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetDruckerListe(REQDruckerListe& oREQ, BSTCont<CString>& druckerCont)
{
   BSASSERT(!oREQ.dokument.IsEmpty());
   druckerCont.Clear();
   BZBoBase oBDok(new DOKUMENT);
   oBDok.SetString(DOKUMENT::DOKUMENTART, oREQ.dokument);

   try
   {
      oBDok.Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("GetDruckerListe: Dokument %1 nicht gefunden."),
               BsaId::DRUCKER_NOTFOUND).Arg(oREQ.dokument);
      }
      throw;
   }
   if (!oBDok.GetBoolNull(DOKUMENT::ISTAGEND).Get())
   {
      BSTHROW BZException(i18n("GetDruckerListe: Dokument %1 fr Tagesenddruck nicht vorgesehen."),
         BsaId::DRUCKER_NOTFOUND).Arg(oREQ.dokument);
   }
   CString relevanterkontext = oBDok.GetString(DOKUMENT::RELEVANTERKONTEXT);

   BZBoList bolist(new BZBoBase(new DRUCKERZUORDTAGEND), new DRUCKERZUORDTAGEND::GetDruckerListe());
  
   bolist.Param()->SetString(DRUCKERZUORDTAGEND::GetDruckerListe::Param::DOKUMENT_EQ, oREQ.dokument);
   if(relevanterkontext == L"MANDANT")
   {
      bolist.Param()->SetIntNull(DRUCKERZUORDTAGEND::GetDruckerListe::Param::NUR_MANDANTNR_EQ, 
                                       oREQ.mModul.mandantnr);
   }
   else if(relevanterkontext == L"ABEREICH")
   {
      bolist.Param()->SetIntNull(DRUCKERZUORDTAGEND::GetDruckerListe::Param::MANDANTNR_EQ, 
                                       oREQ.mModul.mandantnr);
      bolist.Param()->SetIntNull(DRUCKERZUORDTAGEND::GetDruckerListe::Param::MODULNR_EQ, 
                                       oREQ.mModul.modulnr);
   }

   bolist.Read();

   for(int i=0; i<bolist.Count(); i++)
      druckerCont.Insert(new CString(bolist.GetBO(i)->GetString(DRUCKERZUORDTAGEND::DRUCKERBEZ)));

}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::RefreshChache(const BZCollectionId& colId)
{
   BSPtr<ZDDataStruct> pZD  (new BETRIEBDB);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new BETRIEBDB::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();

   CString env = ZDTsGlobal::GetEnv(L"KFZBINDIR");
   if (env.IsEmpty())
   {
      BSTHROW BSException (i18n("Env-Variable KFZBINDIR nicht gesetzt."));
   }
   env += L"/pushevent";

   int sVpdid = -1;
   for(int i=0; i<bolist.Count(); i++)
   {
      if(bolist.GetBO(i)->GetLongNull(BETRIEBDB::VPDID_).Get(0) != sVpdid)
      {
         ArgCont arg(env);
         arg.Add("-eCACHESERVICE_REQUEST");
         CString msg = L"-m" + colId.AsString(); 
         arg.Add(msg);
         CString betnr = L"-d";
         betnr += bolist.GetBO(i)->GetString(BETRIEBDB::BETNR);
         arg.Add(betnr);

         C2SystemRead cmd;
         CCString strErr;
         CCString strOut;

         int ret = cmd.Exec(arg, strOut, strErr);
         if(ret)
         {
            CCString out = strOut;
            out += "\n";
            out += strErr;
            out += "\n";
            BSLog(BS_ERR, i18n("Fehler ausfhren von pushevent %1")).Arg(out);
         }
      }
      sVpdid = bolist.GetBO(i)->GetLongNull(BETRIEBDB::VPDID_).Get(0);
   }
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::IsDokumentKassabuchRelevant(const CString& dokumentart)
{
   BSPtr<ZDDataStruct> pZD(new DOKUMENT);
   BSPtr<BZBoBase>     pB (new BZBoBase(pZD));
   pB->SetString(DOKUMENT::DOKUMENTART, dokumentart);
   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("Dokumentart:%1 nicht gefunden."),
            BsaId::DOKUMENT_NOTFOUND).Arg(dokumentart);
      }
      else
         throw;
   }
   return pB->GetBoolNull(DOKUMENT::KASSABUCHRELEVANT).Get(false);
}

////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetUstCodeFromNtoMwstBetrag(const double& nto_betrag, const double& mwst_betrag)
{
   double calc_UstSatz = (mwst_betrag / nto_betrag) * 100;

   CString ustCode = GetUstCodeFromUstSatz(calc_UstSatz);

   if(ustCode.IsEmpty())
      BSTHROW BZException(i18n("Keine passende Umsatzsteuer gefunden. Ustsatz: %1"), BsaId::UMSATZSTEUER_NOTFOUND).Arg(calc_UstSatz);

   return ustCode;
}
////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetUstCodeFromUstSatz(const double& ustSatz)
{
   BSPtr<ZDDataStruct> pZD(new UMSATZSTEUER);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new UMSATZSTEUER::ListeUstCode);
   BZBoList boList(pB, pList);

   boList.Param()->SetDate(UMSATZSTEUER::ListeUstCode::Param::DATUM, BSDate::CurrentDate());
   boList.Param()->SetBool(UMSATZSTEUER::ListeUstCode::Param::DEFAULTCODE, true);
   boList.Read();

   double diff2CrossUst = csIni->GetValueDouble(L"USTSATZ_MAX_TOLERANZWERT", L"EINSTELLUNGEN");

   CString ustCode;
   for(int iter = 0; iter < boList.Count(); iter++)
   {
      BZBoBase* pBoUst = boList.GetBO(iter);

      if(pBoUst->IsNull(UMSATZSTEUER::USTSATZ))
         continue;

      if(fabs(pBoUst->Get(UMSATZSTEUER::USTSATZ).Get(0) - ustSatz) < diff2CrossUst)
      {
         ustCode = pBoUst->Get(UMSATZSTEUER::USTCODE);
         diff2CrossUst = fabs(pBoUst->Get(UMSATZSTEUER::USTSATZ).Get(0) - ustSatz);
      }
   }
   return ustCode;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetListeGueltigeBetriebe(BSTCont<CString>& ans)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD(new ARBBEREICH);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ARBBEREICH::ListeBetriebe);
   BZBoList boList(pB, pList);
   boList.Read();

   for(int i=0; i<boList.Count(); i++)
      ans.Insert(new CString(boList.GetBO(i)->GetString(ARBBEREICH::BETNR)));
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetListeGueltigeBetriebe(BSMap<CString, MModul>& oANSMap)
{
   BSPtr<ZDDataStruct> pZD(new ARBBEREICH);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ARBBEREICH::ListeBetriebe);
   BZBoList boList(pB, pList);
   boList.Read();

   for(int i=0; i<boList.Count(); i++)
   {
      MModul* pMM = new MModul(boList.GetBO(i)->GetIntNull(ARBBEREICH::MANDANTNR),
                               boList.GetBO(i)->GetIntNull(ARBBEREICH::MODULNR));

      oANSMap.Insert(boList.GetBO(i)->GetString(ARBBEREICH::BETNR), pMM);
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetListeGueltigeMandanten(BSTCont<int>& ans)
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD(new ARBBEREICH);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ARBBEREICH::Mandanten);
   BZBoList boList(pB, pList);
   boList.Param()->SetBoolNull(ARBBEREICH::Mandanten::Param::NURGUELTIGE_EQ, true);
   boList.Param()->SetString(ARBBEREICH::Mandanten::Param::V_NW, csIni->GetValue(L"NW",L"MODUL"));
   boList.Param()->SetString(ARBBEREICH::Mandanten::Param::V_GW, csIni->GetValue(L"GW",L"MODUL"));
   boList.Param()->SetString(ARBBEREICH::Mandanten::Param::V_KD, csIni->GetValue(L"KD",L"MODUL"));
   boList.Read();

   for(int i=0; i<boList.Count(); i++)
      ans.Insert(new int(boList.GetBO(i)->GetIntNull(ARBBEREICH::MANDANTNR).Get(0)));
}
////////////////////////////////////////////////////////////////////////////////
int_null MgrBsa::GetNWHerstellerGarantie(const ReqNWHerstellerGarantie& oReq)
{
   BSTRACE_FCT("MgrBsa::GetNWHerstellerGarantie");

   BSASSERT(oReq.verrfabrikatkz.IsEmpty() == false);

   BSPtr<BZBoBase    > pBo(new BZBoBase(new HERSTELLERGARANTIE));
   BSPtr<ZDListAccess> pListAcc(new HERSTELLERGARANTIE::Liste());
   BSPtr<BZBoList    > pBoList(new BZBoList(pBo, pListAcc));

   pBoList->Param()->SetString(HERSTELLERGARANTIE::Liste::Param::VERRFABRIKATKZ_EQ, oReq.verrfabrikatkz);
   pBoList->Param()->SetString(HERSTELLERGARANTIE::Liste::Param::MODCODE_EQ       , oReq.modellcode);

   pBoList->Read();

   int_null garantiedauer;
   bool     modcode_found = false;
   int      strlen_modcodeUsed = 0;
   for(int iter = 0; iter < pBoList->Count(); iter++)
   {
      BZBoBase* pBoIter     = pBoList->GetBO(iter);
      CString   modcodeIter = pBoIter->GetString(HERSTELLERGARANTIE::MODCODE);

      if( modcodeIter.IsEmpty())
      {
         if(!modcode_found)
            garantiedauer = pBoIter->GetIntNull(HERSTELLERGARANTIE::GARANTIEMONATE);
      }
      else
      {
         if(modcodeIter == oReq.modellcode)
         {
            garantiedauer = pBoIter->GetIntNull(HERSTELLERGARANTIE::GARANTIEMONATE);
            modcode_found = true;
            break;
         }
         else
         {
            int strlen_modcodeIter = modcodeIter.GetLength();
            if(strlen_modcodeIter > strlen_modcodeUsed)
            {
               if(modcodeIter == oReq.modellcode.Mid(0, strlen_modcodeIter))
               {
                  modcode_found = true; 
                  strlen_modcodeUsed = strlen_modcodeIter;

                  garantiedauer = pBoIter->GetIntNull(HERSTELLERGARANTIE::GARANTIEMONATE);
               }
            }
         }
      }
   }

   return garantiedauer;
}
////////////////////////////////////////////////////////////////////////////////
double_null MgrBsa::GetStempelmarkeKTS(const double_null& betrag)
{
   return GetStempelmarke(betrag, csIni->GetValue(L"KTS", L"MODULGRP"));
}
////////////////////////////////////////////////////////////////////////////////
double_null MgrBsa::GetStempelmarkeNGS(const double_null& betrag)
{
   return GetStempelmarke(betrag, csIni->GetValue(L"NGS", L"MODULGRP"));
}
////////////////////////////////////////////////////////////////////////////////
double_null MgrBsa::GetStempelmarke(const double_null& betrag, const CString& modulGrp)
{
   if(betrag.IsNull())
      return double_null();

   BSPtr<ZDDataStruct> pZD(new STMGRENZEN());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new STMGRENZEN::Liste);
   BZBoList boList(pB, pList);
   boList.Param()->SetString(STMGRENZEN::Liste::Param::MODULGRP_EQ, modulGrp);

   boList.Read();

   for(int i=0; i<boList.Count(); i++)
   {
      //Es wird jener BetragBis ermittelt der grer oder gleich dem bergebenen Betrag ist.
      //BSCompare=> returns -1  wenn x <  y
      if(BSCompare(boList.GetBO(i)->GetDoubleNull(STMGRENZEN::BETRAGBIS).Get(), betrag.Get()) < 0)
         continue;
      else
         return boList.GetBO(i)->GetDoubleNull(STMGRENZEN::BETRAGSTM);
   }
   //wenn nix gefunden=>NULL
   return double_null();
}
////////////////////////////////////////////////////////////////////////////////
long_null MgrBsa::GetVerkSteuerBeh(long lfdnrEinkSteuerBeh, MModul oMM)
{
   BSPtr<BZBoBase> pB(new BoSteuerBehandlung());
   BSPtr<ZDListAccess> pList(new STEUERBEHANDLUNG::ListeVerkSteuerBeh);
   BZBoList boList(pB, pList);
   boList.Param()->SetLong(STEUERBEHANDLUNG::ListeVerkSteuerBeh::Param::LFDNREINKAUF_EQ, lfdnrEinkSteuerBeh);
   boList.Read();

   long_null steuerBeh;
   if(boList.Count() == 0)
      steuerBeh=long_null();
   else if(boList.Count() == 1)
      steuerBeh = boList.GetBO(0)->GetLongNull(STEUERBEHANDLUNG::LFDNR);
   else
   {
      REQModul oREQModul;
      AnsArbBereich oAnsArbBereich;
      oREQModul.mandantnr = oMM.mandantnr;
      oREQModul.modulnr   = oMM.modulnr;
      GetArbeitsbereich(oREQModul, oAnsArbBereich);
      for(int i=0; i<boList.Count(); i++)
      {
         if(boList.GetBO(i)->GetBoolNull(STEUERBEHANDLUNG::GUELTIGFUERNW).Get(false) &&
            csIni->GetNeutralKey(oAnsArbBereich.modulkz, L"MODUL") == L"NW")
           steuerBeh = boList.GetBO(i)->GetLongNull(STEUERBEHANDLUNG::LFDNR);
         else if(boList.GetBO(i)->GetBoolNull(STEUERBEHANDLUNG::GUELTIGFUERGW).Get(false) &&
            csIni->GetNeutralKey(oAnsArbBereich.modulkz, L"MODUL") == L"GW")
           steuerBeh = boList.GetBO(i)->GetLongNull(STEUERBEHANDLUNG::LFDNR);
      }
   }
   return steuerBeh;
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::IsKTSStandortUnique(const MModul& req, CString& standort)
{
   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new KTSSTANDORT::Liste);
   BZBoList boList(pB, pList);

   boList.Param()->SetInt     (KTSSTANDORT::Liste::Param::MANDANTNR_EQ, req.mandantnr.Get() );
   boList.Param()->SetIntNull (KTSSTANDORT::Liste::Param::MODULNR_EQ  , req.modulnr.Get()   );
   boList.Param()->SetBoolNull(KTSSTANDORT::Liste::Param::LOESCHKZ_EQ , false               );
   boList.Read();

   if (boList.Count() == 1)
   {
      standort = boList.GetBO(0)->GetString(KTSSTANDORT::STANDORT);
      return true; 
   }
   else if (boList.Count() == 0) 
   {
      BSTHROW BZException(i18n("Es wurde kein Standort zum Arbeitsbereich (Mand %1/Mod %2) gefunden."),
         BsaId::KTSSTANDORT_NOTFOUND).Arg(req.mandantnr).Arg(req.modulnr);
      return false;  
   }
   return false;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetKTSStandortListe(BSTCont<AnsKtsStandort>& ans, 
                                 const MModul& req, const bool_null& loeschkz )
{
   ans.Clear();

   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new KTSSTANDORT::Liste);
   BZBoList boList(pB, pList);

   boList.Param()->SetInt     (KTSSTANDORT::Liste::Param::MANDANTNR_EQ, req.mandantnr.Get() );
   boList.Param()->SetIntNull (KTSSTANDORT::Liste::Param::MODULNR_EQ  , req.modulnr.Get()   );
   boList.Param()->SetBoolNull(KTSSTANDORT::Liste::Param::LOESCHKZ_EQ , loeschkz            );
   boList.Read();

   AnsKtsStandort* ansstandort;
   int count = boList.Count();
   for(int i=0; i<count; i++)
   {
      ansstandort = new AnsKtsStandort;
      ansstandort->mandantnr        = boList.GetBO(i)->GetIntNull (KTSSTANDORT::MANDANTNR  ).Get();
      ansstandort->modulnr          = boList.GetBO(i)->GetIntNull (KTSSTANDORT::MODULNR    ).Get();
      ansstandort->standortkz       = boList.GetBO(i)->GetString  (KTSSTANDORT::STANDORT   );
      ansstandort->bezeichnung      = boList.GetBO(i)->GetString  (KTSSTANDORT::BEZEICHNUNG);
      ansstandort->gptnr            = boList.GetBO(i)->GetLongNull(KTSSTANDORT::GPTNR      );
      ansstandort->loeschkz         = boList.GetBO(i)->GetBoolNull(KTSSTANDORT::LOESCHKZ   ).Get(false);
      ans.Insert(ansstandort);
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetKTSStandort(const MModul& req, const CString& standort,
                            AnsKtsStandort& ans,  bool incl_deleted)
{
   BSASSERT(!standort.IsEmpty());
   BSASSERT(!req.mandantnr.IsNull());
   BSASSERT(!req.modulnr.IsNull());

   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   pB->SetInt     (KTSSTANDORT::MANDANTNR, req.mandantnr.Get() );
   pB->SetIntNull (KTSSTANDORT::MODULNR  , req.modulnr.Get()   );
   pB->SetString  (KTSSTANDORT::STANDORT , standort            );

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
      {
         BSTHROW BZException(i18n("KTS-Standort mit Mand/Mod/Standort %1/%2/%3 nicht gefunden."),
            BsaId::KTSSTANDORT_NOTFOUND).Arg(req.mandantnr.Get()).Arg(req.modulnr.Get()).Arg(standort);
      }
      else
         throw;
   }

   if (incl_deleted == false 
      && pB->GetBoolNull(KTSSTANDORT::LOESCHKZ).Get() == true)
   {
      BSTHROW BZException(i18n("KTS-Standort mit Mand/Mod/Standort %1/%2/%3 ist als gelscht gekennzeichnet."),
         BsaId::KTSSTANDORT_NOTFOUND).Arg(req.mandantnr.Get()).Arg(req.modulnr.Get()).Arg(standort);
   }

   ans.mandantnr        = pB->GetIntNull (KTSSTANDORT::MANDANTNR  ).Get();
   ans.modulnr          = pB->GetIntNull (KTSSTANDORT::MODULNR    ).Get();
   ans.standortkz       = pB->GetString  (KTSSTANDORT::STANDORT   );
   ans.bezeichnung      = pB->GetString  (KTSSTANDORT::BEZEICHNUNG);
   ans.gptnr            = pB->GetLongNull(KTSSTANDORT::GPTNR      );
   ans.loeschkz         = pB->GetBoolNull(KTSSTANDORT::LOESCHKZ   ).Get(false);
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::IsKTSStandortValid(const MModul& req, CString& standort)
{
   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new KTSSTANDORT::Liste);
   BZBoList boList(pB, pList);

   boList.Param()->SetInt     (KTSSTANDORT::Liste::Param::MANDANTNR_EQ, req.mandantnr.Get() );
   boList.Param()->SetIntNull (KTSSTANDORT::Liste::Param::MODULNR_EQ  , req.modulnr.Get()   );
   boList.Param()->SetBoolNull(KTSSTANDORT::Liste::Param::LOESCHKZ_EQ , false               );
   boList.Read();

   if(boList.Count() == 0)
      return false;

   for(int i=0; i < boList.Count(); i++)
   {
      if(boList.GetBO(i)->GetString(KTSSTANDORT::STANDORT) == standort)
         return true;
   }
   
   return false;
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::DeleteStandortSaetze(const MModul& req)
{
   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new KTSSTANDORT::Liste);
   BZBoList boList(pB, pList);

   boList.Param()->SetInt     (KTSSTANDORT::Liste::Param::MANDANTNR_EQ, req.mandantnr.Get() );
   boList.Param()->SetIntNull (KTSSTANDORT::Liste::Param::MODULNR_EQ  , req.modulnr.Get()   );
   boList.Read();

   boList.DeleteEntries();
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::ExistsMehrereKTSStandorte(bool& existsMehrereStandorte)
{
   existsMehrereStandorte = true; // defaultmaessig wird von mehreren Standorten ausgegangen 
   BSPtr<ZDDataStruct> pZD(new KTSSTANDORT());
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new KTSSTANDORT::ListeGesDB);
   BZBoList boList(pB, pList);
   boList.Read();

   if (boList.Count() == 1)
      existsMehrereStandorte = false; 
}

////////////////////////////////////////////////////////////////////////////////
long_null MgrBsa::GetIntGFFromExtGF(const CString& extgfbez, const CString& extsystemname)
{
   BSPtr<ZDDataStruct> pZD  (new EXTNGEGF);
   BSPtr<ZDListAccess> pList(new EXTNGEGF::Liste);
   BSPtr<BZBoBase>     pB   (new BZBoBase(pZD));
   BZBoList bolist(pB, pList);
   bolist.Param()->SetString(EXTNGEGF::Liste::Param::EXT_GFIDENTIFIKATION_EQ   , extgfbez);
   bolist.Param()->SetString(EXTNGEGF::Liste::Param::EXT_SYSTEMNAME_EQ , extsystemname);
   bolist.Read();

   if (bolist.Count() > 1)
      BSTHROW BZException(i18n("Fr den externen Geschftsfall %1 wurde mehr als ein interner Geschftsfall gefunden"),
      BsaId::NGEINTGF_NOTFOUND).Arg(extgfbez);

   if (bolist.Count() == 1)
      return bolist.GetBO(0)->GetLongNull(EXTNGEGF::NGEGFLFDNR).Get();
   else
      return long_null();

}

////////////////////////////////////////////////////////////////////////////////
datetime_null MgrBsa::GetZeitpunktFromProgrammprotokollierung(const ReqProgramProt& oReq)
{
   // Parameter berprfen
   CheckParamForProgrammprotokollierung(oReq);

   // Programmprotokollierung lesen
   BZBoList oBoList(new BZBoBase(new PROGRAMPROTPOS()), new PROGRAMPROTPOS::Liste());
   ReadProgrammprotokollierung(&oBoList, oReq);

   // Return NUll-Type wenn keine Programmprotokollierung gefunden
   if(oBoList.Count() < 1)
      return datetime_null();

   return oBoList.GetBO(0)->Get(PROGRAMPROTPOS::ZEITPKT);
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::SetZeitpunktForProgrammprotokollierung(const ReqProgramProt& oReq, 
                                                    const datetime_null& zeitpunkt)
{
   // berprfung ob Zeitpunkt gesetzt ist
   BSASSERT(!zeitpunkt.IsNull());

   // Parameter berprfen
   CheckParamForProgrammprotokollierung(oReq);

   // Programmprotokollierung lesen
   BZBoList oBoList(new BZBoBase(new PROGRAMPROTPOS()), new PROGRAMPROTPOS::Liste());
   ReadProgrammprotokollierung(&oBoList, oReq);

   // Neuanlage falls keine Programmprotokollierung gefunden wurde
   if(oBoList.Count() < 1)
   {
      BSPtr<BZBoBase> pBo(new BZBoBase(new PROGRAMPROTPOS()));
      pBo->SetDefaults();
      pBo->Set(PROGRAMPROTPOS::PROGRAMID , oReq.programid    , true);
      pBo->Set(PROGRAMPROTPOS::MANDANTNR , oReq.mandantnr    , true);
      pBo->Set(PROGRAMPROTPOS::MODULNR   , oReq.modulnr      , true);
      pBo->Set(PROGRAMPROTPOS::MANDANTNR , oReq.nur_mandantnr, true);
      pBo->Set(PROGRAMPROTPOS::BETNR     , oReq.betnr        , true);
      pBo->Set(PROGRAMPROTPOS::FABRIKATKZ, oReq.fabrikatkz   , true);
      oBoList.AddBO(pBo);
   }

   // Neuen Zeitpunkt (timestamp) setzen
   oBoList.GetBO(0)->Set(PROGRAMPROTPOS::ZEITPKT, zeitpunkt);

   // Insert bzw. Update des Bo's
   oBoList.GetBO(0)->Persist();
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::CheckParamForProgrammprotokollierung(const ReqProgramProt& oReq)
{
   if(oReq.programid.IsEmpty())
      BSTHROW BZException(i18n("Programm-ID muss gesetzt sein"), BsaId::PROGRAMPROT_PARAMCONFLICT);

   if((!oReq.mandantnr.IsNull() &&  oReq.modulnr.IsNull()) ||
      ( oReq.mandantnr.IsNull() && !oReq.modulnr.IsNull())   )
      BSTHROW BZException(i18n("Wenn Mandant-Nr. bzw. Modul-Nr. gesetzt ist muss Modul-Nr. bzw. Mandant-Nr. auch gesetzt sein"), 
      BsaId::PROGRAMPROT_PARAMCONFLICT);

   // Da vorher berprft wurde dass MandanrNr und ModulNr entweder beide gesetz oder 
   // beide nicht gestzt sein mssen, muss hier nur auf MandantNr berprft werden
   // MModul / nur_mandantnr / betnr
   if(!oReq.mandantnr.IsNull())
   {
      if(!oReq.nur_mandantnr.IsNull() || !oReq.betnr.IsEmpty())
         BSTHROW BZException(i18n("Wenn MModul gesetzt ist darf die Betriebs-Nr. bzw. Nur-Mandant-Nr. nicht gesetzt sein"), 
         BsaId::PROGRAMPROT_PARAMCONFLICT);
   }
   else if(!oReq.nur_mandantnr.IsNull())
   {
      if(!oReq.betnr.IsEmpty())
         BSTHROW BZException(i18n("Wenn Nur-Mandant-Nr. gesetzt ist darf die Betriebs-Nr. bzw. MModul nicht gesetzt sein"), 
         BsaId::PROGRAMPROT_PARAMCONFLICT);
   }

   if(!oReq.fabrikatkz.IsEmpty() && oReq.mandantnr.IsNull())
      BSTHROW BZException(i18n("Wenn Fabrikat gesetzt ist muss auch MModul gesetzt sein"), 
      BsaId::PROGRAMPROT_PARAMCONFLICT);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::ReadProgrammprotokollierung(BZBoList* pBoList, const ReqProgramProt& oReq)
{
   // BZBOList-Pointer muss gesetzt sein
   BSASSERT(pBoList);

   // Kriterien setzen
   pBoList->Param()->SetString( PROGRAMPROTPOS::Liste::Param::PROGRAMID_EQ    , oReq.programid    );
   pBoList->Param()->SetIntNull(PROGRAMPROTPOS::Liste::Param::MANDANTNR_EQ    , oReq.mandantnr    );
   pBoList->Param()->SetIntNull(PROGRAMPROTPOS::Liste::Param::MODULNR_EQ      , oReq.modulnr      );
   pBoList->Param()->SetIntNull(PROGRAMPROTPOS::Liste::Param::NUR_MANDANTNR_EQ, oReq.nur_mandantnr);
   pBoList->Param()->SetString( PROGRAMPROTPOS::Liste::Param::BETNR_EQ        , oReq.betnr        );
   pBoList->Param()->SetString( PROGRAMPROTPOS::Liste::Param::FABRIKATKZ_EQ   , oReq.fabrikatkz   );

   // Liste lesen
   pBoList->Read();

   // Mehr als einen Eintrag gefunden
   if(pBoList->Count() > 1)
      BSTHROW BZException(i18n("Keine eindeutige Programmprotokollierung gefunden."), BsaId::PROGRAMPROT_NOTUNIQUE);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetRechFixText(const ReqFixText& req, BSTCont<AnsFixText>& ans)
{
   //Barverkufe.
   BSPtr<ZDDataStruct> tmpDatastr(new TWERECHFIXTEXT());
   BSPtr<BZBoBase>     tmpBOBase (new BZBoBase(tmpDatastr));
   BSPtr<ZDListAccess> tmplistAcc(new TWERECHFIXTEXT::ListeTexte());
   BZBoList bLTexte(tmpBOBase,tmplistAcc);
   ZDParamStruct* pParam = bLTexte.Param();

   pParam->SetString  (TWERECHFIXTEXT::ListeTexte::Param::AUFTRAGSARTTYP_EQ,  req.auftragsArtTyp);
   pParam->SetString  (TWERECHFIXTEXT::ListeTexte::Param::KOMBIKZ_EQ,         req.kombiKz);
   pParam->SetBoolNull(TWERECHFIXTEXT::ListeTexte::Param::MWSTKUNDE_EQ,       req.mwstKunde);
   pParam->SetString  (TWERECHFIXTEXT::ListeTexte::Param::POSITIONSKZ_EQ,     req.postionsKz);

   bLTexte.Read();

   for (int i = 0; i < bLTexte.Count(); i++)
   {
      BZBoBase* pBO = bLTexte.GetBO(i);
      AnsFixText* pANS = new AnsFixText();

      ans.Insert(pANS);

      CString txt = pBO->GetString  (TWERECHFIXTEXT::TEXT);

      if (txt.Find(L"%1") >= 0)
      {
         BSFormat f(txt);
         f.Arg(req.anzeigeBetrag.Get(0));
         txt = f.Result();
      }

      pANS->auftragsArtTyp = pBO->GetString  (TWERECHFIXTEXT::AUFTRAGSARTTYP);
      pANS->kombiKz        = pBO->GetString  (TWERECHFIXTEXT::KOMBIKZ);
      pANS->mwstKunde      = pBO->GetBoolNull(TWERECHFIXTEXT::MWSTKUNDE);
      pANS->postionsKz     = pBO->GetString  (TWERECHFIXTEXT::POSITIONSKZ);
      pANS->text           = txt;
      pANS->sortierung     = pBO->GetIntNull (TWERECHFIXTEXT::SORTIERUNG);
   }
}

////////////////////////////////////////////////////////////////////////
void MgrBsa::GetUstCodes4SteuerBeh(ReqUstCodes4SteuerlBeh& req, AnsUstCodes4SteuerlBeh& ans)
{
   // UStCode aus Standardwerten zunchst holen:

   MgrBsa::AnsStdwAllgemein ansStd;
   GetStandardwerte(ansStd);
   //if (!ansStd.ustcode.IsEmpty()) <- kann nicht Empty sein, da sonst schon "GetStandardwerte()" Exception wirft!
   ans.ustcode_aus_standardwerte = ansStd.ustcode;
      
    // Wenn keine steuerl. Behandlung (LfdNr) bergeben wurde, ist hier auch schon Ende!
   if (req.lfdnr.IsNull())
      return;

   // Jetzt Daten zur steuerliche Behandlung lesen...
   BSPtr<BZBoBase> pBoSteuerBeh(new BoSteuerBehandlung());
   BZBoBase* pBoUstSatz;
   pBoSteuerBeh->SetLongNull(STEUERBEHANDLUNG::LFDNR, req.lfdnr.Get());
   pBoSteuerBeh->ReadAll();

   BZBoDepList* pDLUstSaetze = pBoSteuerBeh->GetBoDepList(STEUERBEHANDLUNG_ID::DP_USTSAETZE);
   
   if (pBoSteuerBeh->GetBoolNull(STEUERBEHANDLUNG::USTUEBERSTKZ).Get(false)) // bersteuert-Kennzeichen gesetzt?
      ans.ustcode_uebersteuert = pBoSteuerBeh->GetString(STEUERBEHANDLUNG::USTUEBERSTCODE);
   if (pDLUstSaetze->Count() == 1) // Wenn nur ein Satz in Liste enthalten ist, ...
   {
      pBoUstSatz = pDLUstSaetze->GetBO(0); // ... dann diesen Satz verwenden!
      ans.ustcode_aus_liste = pBoUstSatz->GetString(STBEHANDLUST::USTCODE);
   }
}
////////////////////////////////////////////////////////////////////////////////
double_null MgrBsa::GetMwStProz(const MModul& oMM, const long_null& gptnr)
{
    // UStProz holen; Zunchst steuerliche Behandlung von GP holen, falls bergeben wurde
    CString ustCode;
    IMgrBsa::ReqUstCodes4SteuerlBeh requst;
    if (!gptnr.IsNull())
    {
        IBoGeschpartnerMModul::ANSKundeMand anskd;
        try 
        {
            IBoGeschpartnerMModul::Create(gptnr,oMM)->GetKundeMand(anskd);
            if (anskd.steuerlbeh>0)
                requst.lfdnr     = anskd.steuerlbeh;
        }
        catch(BSException&)
        { // kein Mandantensegment vorhanden 
        }
    }

    // Effektiven USt-Prozentsatz ermitteln:
    // GetUstCodes4SteuerBeh liefert:
    // Ohne steuerl. Behandlung
    // - UStCode aus den Standardwerten zurckgegeben (in "ustcode_aus_standardwerte").
    // Mit steurl. Behandlung
    // - bersteuerter UStCode (in "ustcode_uebersteuert") oder
    // - UstCode aus der Liste der steuerlichen Behandlung (in "ustcode_aus_liste").
    IMgrBsa::AnsUstCodes4SteuerlBeh ansust;
    GetUstCodes4SteuerBeh(requst, ansust);

    if (!ansust.ustcode_uebersteuert.IsEmpty())
        ustCode = ansust.ustcode_uebersteuert;
    else if (!ansust.ustcode_aus_liste.IsEmpty())
        ustCode = ansust.ustcode_aus_liste;
    else if (!ansust.ustcode_aus_standardwerte.IsEmpty())
        ustCode = ansust.ustcode_aus_standardwerte;

    // Wenn UStCode vorhanden, dann jetzt den Prozentsatz ermitteln:
    if (!ustCode.IsEmpty())
    {
        IMgrBsa::AnsUst ans;
        IMgrBsa::ReqUst req;
        req.ustcode = ustCode;
        req.vondatum = BSDate::CurrentDate();
        GetUst(req, ans);
        return ans.ustprozsatz;
    }
    return double_null();
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::SetMigrationIsRunning(bool migrationIsRunning)
{
   if (migrationIsRunning)
      ZDTsGlobal::SetEnv(L"MIGRATION_IS_RUNNING", L"TRUE", true);
   else
      ZDTsGlobal::SetEnv(L"MIGRATION_IS_RUNNING", L"", true);
}
////////////////////////////////////////////////////////////////////////////////
bool MgrBsa::GetMigrationIsRunning()
{
   CString val = ZDTsGlobal::GetEnv(L"MIGRATION_IS_RUNNING");
   return (val == L"TRUE");
}
////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetAbnehmerGrpBHCode(const CString& sModulKz, const CString& sAbnehmerGrp, const CString& sFinanzierung)
{
   BSPtr<ZDDataStruct> pZD(new ABNEHMERGRPBHCODE);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));

   CString sFin = sFinanzierung;
   if(sFin.IsEmpty())
      sFin = csIni->GetValue(L"KEI", L"FINANZIERUNGSKZ");

   pB->SetString(ABNEHMERGRPBHCODE::MODULKZ,         sModulKz);
   pB->SetString(ABNEHMERGRPBHCODE::ABNEHMERGRUPPE,  sAbnehmerGrp);
   pB->SetString(ABNEHMERGRPBHCODE::FINANZIERUNGSKZ, sFin);

   try
   {
      pB->Read();
   }
   catch(BZException& oBZException)
   {
      if(oBZException.GetId() == BZExceptionId::RECORD_NOT_FOUND)
         return L"";
      else
         throw;
   }

   return pB->GetString(ABNEHMERGRPBHCODE::BHCODE);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetAbnehmerGrpBHCodeListe(BSTCont<AnsAbnehmerGrpBHCode>& ans)
{
   BSPtr<ZDDataStruct> pZD(new ABNEHMERGRPBHCODE);
   BSPtr<BZBoBase> pB(new BZBoBase(pZD));
   BSPtr<ZDListAccess> pList(new ABNEHMERGRPBHCODE::Liste);
   BZBoList bolist(pB, pList);
   bolist.Read();

   AnsAbnehmerGrpBHCode* ansAb;
   const BZBoBase* pBo;
   int count = bolist.Count();
   for(int i=0; i<count; i++)
   {
      pBo = bolist.GetBO(i);
      ansAb = new AnsAbnehmerGrpBHCode;
      ans.Insert(ansAb);
      ansAb->modulkz         = pBo->GetString(ABNEHMERGRPBHCODE::MODULKZ);
      ansAb->abnehmergrp     = pBo->GetString(ABNEHMERGRPBHCODE::ABNEHMERGRUPPE);
      ansAb->finanzierungskz = pBo->GetString(ABNEHMERGRPBHCODE::FINANZIERUNGSKZ);
      ansAb->bhcode          = pBo->GetString(ABNEHMERGRPBHCODE::BHCODE);
   }
}

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::FillVerrechnungsart(const BZBoBase* pBo, AnsVerrechnungsart* oAns, const bool bInclDel/*=false*/)
{
   // Wenn keine lschmarkierten und Verrechnugnsart lschmarkiert nichts machen
   if (!bInclDel && pBo->GetBoolNull(VERRECHNUNGSART::LOESCHKZ).Get(false))
      return;

   oAns->abteilungkz = pBo->Get(VERRECHNUNGSART::ABTEILUNGKZ);
   oAns->bezeichnung = pBo->Get(VERRECHNUNGSART::BEZEICHNUNG);
   oAns->bhcode      = pBo->Get(VERRECHNUNGSART::BHCODE     );
   oAns->etapkz      = pBo->Get(VERRECHNUNGSART::ETAPKZ     );
   oAns->loeschkz    = pBo->GetBoolNull(VERRECHNUNGSART::LOESCHKZ ).Get(true);
   oAns->mandantnr   = pBo->GetIntNull (VERRECHNUNGSART::MANDANTNR).Get(-1)  ;
   oAns->modulnr     = pBo->GetIntNull (VERRECHNUNGSART::MODULNR  ).Get(-1)  ;
   oAns->verrechart  = pBo->Get(VERRECHNUNGSART::VERRECHART );
}// FillVerrechnungsart

////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetVerrechnungsartListe(const MModul& oMModul, BSTCont<AnsVerrechnungsart>& contAns, const bool bInclDel/*=false*/)
{
   BSPtr<BZBoBase>     pB   (new BZBoBase(new VERRECHNUNGSART()));
   BSPtr<ZDListAccess> pList(new VERRECHNUNGSART::Liste);
   BZBoList bolist(pB, pList);
   bolist.Param()->SetInt(VERRECHNUNGSART::Liste::Param::MANDANTNR_EQ, oMModul.mandantnr.Get(-1));
   bolist.Param()->SetInt(VERRECHNUNGSART::Liste::Param::MODULNR_EQ  , oMModul.modulnr.Get(-1)  );
   bolist.Read();

   for (int i=0; i < bolist.Count(); i++)
   {
      const BZBoBase*     pBo = bolist.GetBO(i);
      AnsVerrechnungsart* oAns = new AnsVerrechnungsart();

      contAns.Insert(oAns);
      FillVerrechnungsart(pBo, oAns, bInclDel);
   }// for
}// GetVerrechnungsartListe

////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetBetNrFromBetrieb2Betrieb(const CString& ext_systemname, const long_null& gptnrbetrieb, const CString& def)
{
   if(ext_systemname.IsEmpty() || gptnrbetrieb.IsNull())
      return def;

   BZBoList oBoList(new BoBetrieb2Betrieb(), new BETRIEB2BETRIEB::Liste());
   oBoList.Param()->SetString(  BETRIEB2BETRIEB::Liste::Param::EXT_SYSTEMNAME_EQ, ext_systemname);
   oBoList.Param()->SetLongNull(BETRIEB2BETRIEB::Liste::Param::GPTNRBETRIEB1_EQ , gptnrbetrieb  );
   oBoList.Read();

   // Kein gltiges Mapping gefunden ...
   if(oBoList.Count() != 1)
      return def;

   return oBoList.GetBO(0)->Get(BETRIEB2BETRIEB::BETNR2);
}
////////////////////////////////////////////////////////////////////////////////
long_null MgrBsa::GetGptNrFromBetrieb2Betrieb(const CString& ext_systemname, const long_null& gptnrbetrieb, const long_null& def)
{
   if(ext_systemname.IsEmpty() || gptnrbetrieb.IsNull())
      return def;

   BZBoList oBoList(new BoBetrieb2Betrieb(), new BETRIEB2BETRIEB::Liste());
   oBoList.Param()->SetString(  BETRIEB2BETRIEB::Liste::Param::EXT_SYSTEMNAME_EQ, ext_systemname);
   oBoList.Param()->SetLongNull(BETRIEB2BETRIEB::Liste::Param::GPTNRBETRIEB1_EQ , gptnrbetrieb  );
   oBoList.Read();

   // Kein gltiges Mapping gefunden ...
   if(oBoList.Count() != 1)
      return def;

   return oBoList.GetBO(0)->Get(BETRIEB2BETRIEB::GPTNRBETRIEB2);
}

////////////////////////////////////////////////////////////////////////////////
void IMgrBsa::Validate(CString& val, const ZDFieldinfo* pZDFieldinfo, const bool truncate)
{
   BSASSERT(pZDFieldinfo);

   int maxlen = pZDFieldinfo->GetStringLen();
   if (val.GetLength() > maxlen)
   {
      if (truncate)
      {
         val = val.Left(maxlen);
      }
      else
      {
         BSTHROW BSException(i18n("%1: Der Text ist zu lang (max %2 zugelassen >>%3<<)"))
            .Arg(pZDFieldinfo->GetI18n()).Arg(maxlen).Arg(val);
      }
   }
}
////////////////////////////////////////////////////////////////////////////////
CString MgrBsa::GetUserId()
{
   //Verwendung von ZDTsGlobal statt BZGlobal weil es in Serverprozessen
   //ev. kein BZGlobal gibt und BZGlobal::Instance assertet wenn es keins gibt
   if(!ZDTsGlobal::UserInfoDefined())
      return L"";
   return ZDTsGlobal::GetUserInfo().GetUserId();
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::FillMarkenlogos( ANSDruckerForm& ans, const BZBoBase* pBo )
{
   ans.markenlogo_image_lfdnr      = pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE        );
   ans.markenlogo_image_bez        = pBo->Get(DRUCKERFORMULAR::IMAGE5                  );
   ans.markenlogo_image_filename   = pBo->Get(DRUCKERFORMULAR::FILE5                   );

   if(!ans.markenlogo_image_filename.IsEmpty()   && !pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE5).Get(false))
      AddKFZDEALERTOPDIRIfIsRelative(ans.markenlogo_image_filename);

   ans.markenlogos.clear();

   if(pBo->Get(DRUCKERFORMULAR::MARKENLOGO_ANDRUCK).Get(false) && pBo->Get(DRUCKERFORMULAR::MEHRMARKEN).Get(false))
   {
      AddLogo(ans, pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE_1), pBo->Get(DRUCKERFORMULAR::IMAGE_M1), pBo->Get(DRUCKERFORMULAR::FILE_M1), pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE_M1));
      AddLogo(ans, pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE_2), pBo->Get(DRUCKERFORMULAR::IMAGE_M2), pBo->Get(DRUCKERFORMULAR::FILE_M2), pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE_M2));
      AddLogo(ans, pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE_3), pBo->Get(DRUCKERFORMULAR::IMAGE_M3), pBo->Get(DRUCKERFORMULAR::FILE_M3), pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE_M3));
      AddLogo(ans, pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE_4), pBo->Get(DRUCKERFORMULAR::IMAGE_M4), pBo->Get(DRUCKERFORMULAR::FILE_M4), pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE_M4));
      AddLogo(ans, pBo->Get(DRUCKERFORMULAR::MARKENLOGO_IMAGE_5), pBo->Get(DRUCKERFORMULAR::IMAGE_M5), pBo->Get(DRUCKERFORMULAR::FILE_M5), pBo->Get(DRUCKERFORMULAR::ISSTANDARDIMAGE_M5));

      // Fr Sales... Die (in Wartung) oberste Image wird geliefert... Muss eine mindestens existieren.
      if(ans.markenlogos.size()<1)
         BSTHROW BSException (i18n("In der Wartung Markenauftritt wurde kein Markenlogo ausgewhlt."));

      ans.markenlogo_image_lfdnr      = ans.markenlogos.front().image_lfdnr;
      ans.markenlogo_image_bez        = ans.markenlogos.front().image_bez;
      ans.markenlogo_image_filename   = ans.markenlogos.front().image_filename;
   }

   GetImageSize(GetImagePath(ans.markenlogo_image_filename), ans.markenlogo_image_width, ans.markenlogo_image_height);
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::AddLogo( ANSDruckerForm& ans, const long_null& markenlogo_image_lfndr, const CString& markenlogo_image_bez, const CString& markenlogo_image_filename, const bool_null& stdimage )
{
   if(!markenlogo_image_lfndr.IsNull())
   {
      MarkenLogo logo;

      logo.image_lfdnr    = markenlogo_image_lfndr;
      logo.image_bez      = markenlogo_image_bez;
      logo.image_filename = markenlogo_image_filename;

      if(!logo.image_filename.IsEmpty() && !stdimage.Get(false))
         AddKFZDEALERTOPDIRIfIsRelative(logo.image_filename);

      GetImageSize(GetImagePath(logo.image_filename), logo.image_width, logo.image_height);

      ans.markenlogos.push_back(logo);
      ans.markenlogos_ordered.push_back(logo);
   }
   else
   {
      MarkenLogo logo;
      ans.markenlogos_ordered.push_back(logo);
   }
}
////////////////////////////////////////////////////////////////////////////////
void MgrBsa::GetDefaultZahlungsziel(long_null& lZahlungsZiel, CString& sZahlungsZielArt)
{
   BZBoList oBoLZahlungsziel(new BoZahlungsziel(), new ZAHLUNGSZIEL::ListeIsDefault());

   oBoLZahlungsziel.Read();

   if (oBoLZahlungsziel.Count() == 0 || oBoLZahlungsziel.Count() > 1)
   {
      lZahlungsZiel    = long_null();
      sZahlungsZielArt = L"";
   }
   else
   {
      lZahlungsZiel    = oBoLZahlungsziel.GetBO(0)->Get(ZAHLUNGSZIEL::LFDNR      );
      sZahlungsZielArt = oBoLZahlungsziel.GetBO(0)->Get(ZAHLUNGSZIEL::ZAHLUNGSART);
   }
}
////////////////////////////////////////////////////////////////////////////////

void MgrBsa::CreateOrUpdateServiceVertrag(ReqServiceBrandContract& oReqMarkenVertrag)
{
   // Assertion auf muss Felder
   // aus dem VerFab das Fabrikat ermitteln
   // prfen, ob des fr das Fabrikat und BetriebsNr die BetnrZuord gibt
   //    - wenn nicht anlegen mit fab, betnr, impbetnr und werksbetnr
   //    - wenn es die zuordnung gibt dann bei der BetnrZuord die impbetnr und werksbetnr updaten
   // prfen, ob es bereits einen markenvertrag fr diese Betnrzuord und VerFab gibt
   //    - wenn ja, dann von und bis Datum updaten (falls unterschiedlich)
   //    - wenn nicht fr diese BetnrZuord und VerrFab einen neuen Markenvertrag anlegen

   BSASSERT(!oReqMarkenVertrag.betriebsNr.IsEmpty());
   BSASSERT(!oReqMarkenVertrag.importeurBetNr.IsEmpty());
   BSASSERT(!oReqMarkenVertrag.verrFabrikatKz.IsEmpty());
   BSASSERT(!oReqMarkenVertrag.werksBetNr.IsEmpty());

   MgrFabrikat::AnsVerrFabrikat ans;
   MgrFabrikat::Create()->GetVerrFabrikat(oReqMarkenVertrag.verrFabrikatKz, ans);

   BSASSERT(!ans.fabrikatkz.IsEmpty());

   BSPtr<BZBoBase> pBoBetNrZuord(new BoBetNrZuord());
   CString betNr = oReqMarkenVertrag.betriebsNr;
   betNr.TrimLeft(L"0");
   pBoBetNrZuord->SetString(BETNRZUORD::BETNR,             betNr);
   pBoBetNrZuord->SetString(BETNRZUORD::FABRIKATKZ,        ans.fabrikatkz);

   if(!pBoBetNrZuord->ReadIfExists())
   {  
      BSTCont<IMgrBetrieb::ANSBetrieb> oContInstallierteBetriebe;
      IMgrBetrieb::Create()->GetListeInstallierteBetriebe(oContInstallierteBetriebe, false, false, true);
     
      // Prfen, ob Betriebsnummer in BetMandModul gltig ist
      bool found = false;
      for(int i = 0; i < oContInstallierteBetriebe.Count(); i++)
      {
         CString tempBetNr = oContInstallierteBetriebe.GetObj(i)->betnr;
         tempBetNr.TrimLeft(L"0");
         if(tempBetNr == betNr)
         {
            found = true;
            break;
         }
      }
      if(!found)
         BSTHROW(BSException(L"No valid Dealer-Number: %1").Arg(betNr));

      pBoBetNrZuord->SetState(BZBoBase::ISNEW);
   }
   else
      pBoBetNrZuord->ReadAll();

   pBoBetNrZuord->SetString(BETNRZUORD::WERKSBETRIEBSNR,   oReqMarkenVertrag.werksBetNr);
   pBoBetNrZuord->SetString(BETNRZUORD::FABBETNR,          oReqMarkenVertrag.importeurBetNr);

   BZBoDepList* pBlMarkenVertrag = pBoBetNrZuord->GetBoDepList(MARKENVERTRAEGE_ID::DP_MARKENVERTRAEGE);
   int index = pBlMarkenVertrag->GetIndex(MARKENVERTRAG::BETNR, betNr,
                                          MARKENVERTRAG::FABRIKATKZ, ans.fabrikatkz,
                                          MARKENVERTRAG::VERRFABRIKATKZ, ans.verrfabrikat);
   if(index < 0)
   {
      BSPtr<BZBoBase> pBoMarkenvertrag(pBlMarkenVertrag->NewBO().release());
      pBoMarkenvertrag->SetState(BZBoBase::ISNEW);
      pBoMarkenvertrag->SetString(MARKENVERTRAG::BETNR,            betNr);
      pBoMarkenvertrag->SetString(MARKENVERTRAG::FABRIKATKZ,       ans.fabrikatkz);
      pBoMarkenvertrag->SetString(MARKENVERTRAG::VERRFABRIKATKZ,   ans.verrfabrikat);
      pBoMarkenvertrag->SetDateNull(MARKENVERTRAG::GUELTIG_VON,    oReqMarkenVertrag.vonDatum);
      pBoMarkenvertrag->SetDateNull(MARKENVERTRAG::GUELTIG_BIS,    oReqMarkenVertrag.bisDatum);
      pBlMarkenVertrag->AddBO(pBoMarkenvertrag);
   }
   else
   {
      BZBoBase* pBoMarkenvertrag = pBlMarkenVertrag->GetBO(index);
      if (!pBoMarkenvertrag->GetDateNull(MARKENVERTRAG::GUELTIG_VON).IsEqual(oReqMarkenVertrag.vonDatum) ||
          !pBoMarkenvertrag->GetDateNull(MARKENVERTRAG::GUELTIG_BIS).IsEqual(oReqMarkenVertrag.bisDatum))
      {
         pBoMarkenvertrag->SetDateNull(MARKENVERTRAG::GUELTIG_VON,    oReqMarkenVertrag.vonDatum);
         pBoMarkenvertrag->SetDateNull(MARKENVERTRAG::GUELTIG_BIS,    oReqMarkenVertrag.bisDatum);
      }
   }

   pBoBetNrZuord->PersistAll();
}

//------------------------------------------------------------------------------
// Dateiname liefern und sicherstellen, da Directory existiert.
//------------------------------------------------------------------------------
CString MgrBsa::GetAuswExportFilename(const CString& sNeutralkey,      // zu Range AUSW_EXPORT_DATEINAMEN.
                                      const CString& sFileExtension,   // = L"", Kennung der Datei.
                                      bool           bAddTimestamp,    // = false, Zeitstempel.
                                      const CString& sBetriebsNr,      // = L"", BetriebsNummer.
                                      const CString& nameLabel)        // = L"", Namenskennzeichnung
{
   CString sDirFilename = csIni->GetValue( sNeutralkey, L"AUSW_EXPORT_DATEINAMEN" );
   if (sDirFilename.IsEmpty())
      BSTHROW BSException(i18n("Der Name der Ausgabedatei wurde nicht in den Systemparametern gefunden (%1/%2)."))
            .Arg(sNeutralkey)
            .Arg(L"AUSW_EXPORT_DATEINAMEN");

   CString sDealerPath = ZDTsGlobal::GetEnv(L"KFZDEALERFILESDIR");
   if(sDealerPath.IsEmpty())
      BSTHROW BSException(i18n("KFZDEALERFILESDIR ist nicht gesetzt."));

   // Verzeichnis-Trennzeichen verallgemeinern (sodass beide Trennzeichen in der Konfiguration erkannt werden)
   const CString sSeparator = BSDir::Separator();
   sDirFilename.Replace(L"/", sSeparator);
   sDirFilename.Replace(L"\\", sSeparator);

   // Dealer-Verzeichnis vorne anfgen
   sDirFilename = sDealerPath + sSeparator + sDirFilename;

   // BetriebsNr optional hinzufgen
   if (!sBetriebsNr.IsEmpty())
      sDirFilename += L"_" + sBetriebsNr;

   // Namenskennzeichnung optional hinzufgen
   if( !nameLabel.IsEmpty() )
      sDirFilename += L"_" + nameLabel;

   // aktuellen Timestamp optional hinzufgen
   if (bAddTimestamp)
      sDirFilename += L"_" + BSDateTime::CurrentDateTime().AsString(L"%Y%0m%0d%0H%0M%0S");

   // FileExtension optional hinzufgen
   if (!sFileExtension.IsEmpty())
      sDirFilename += L"." + sFileExtension;

   // Pfad und Dateiname bereinigen und extrahieren
   sDirFilename = BSDir::AbsPath(sDirFilename);
   CString sFilename  = BSDir::Basename(sDirFilename);
   CString sDirectory = sDirFilename.Left(sDirFilename.GetLength() - sFilename.GetLength());

   // wenn Verzeichnis nicht existiert, dieses anlegen
   if (!BSDir::Exists(sDirectory))
   {
      if (!BSDir::Create(sDirectory))
         BSTHROW BSException(i18n("Verzeichnis %1 kann nicht angelegt werden.")).Arg(sDirectory);
   }

   return sDirFilename;
}

//------------------------------------------------------------------------------
void MgrBsa::GetDruckAusgabeKonfig(const CString& dokumentart, DruckAusgabeKonfig& ans)
{
   // Standardkonfiguration
   ans.dokumentart     = dokumentart;
   ans.druckausgabe    = csIni->GetValue(L"C2", L"DRUCKAUSGABE");
   ans.vorschauausgabe = csIni->GetValue(L"C2", L"DRUCKAUSGABE");
   ans.archivierung.SetNull();
   ans.versiondruckxml.SetNull();
   ans.druckexportkanaele.Clear();

   // Falls Dokumentart leer oder nicht gefunden wurde, wird die Standardkonfiguration zurckgeliefert
   if (dokumentart.IsEmpty())
      return;

   BoDruckAusgabeKonfig oBo;
   oBo.Set(DRUCKAUSGABEKONFIG::DOKUMENTART, dokumentart);
   if (oBo.ReadIfExists())
   {
      ans.druckausgabe    = oBo.Get(DRUCKAUSGABEKONFIG::DRUCKAUSGABE   );
      ans.vorschauausgabe = oBo.Get(DRUCKAUSGABEKONFIG::VORSCHAUAUSGABE);
      ans.archivierung    = oBo.Get(DRUCKAUSGABEKONFIG::ARCHIVIERUNG   );
      ans.versiondruckxml = oBo.Get(DRUCKAUSGABEKONFIG::VERSIONDRUCKXML);

      oBo.ReadAll(false);

      for (int itr = 0; itr < oBo.GetBoDepList(DRUCKAUSGABEKONFIG_ID::DP_DRUCKAUSGABEKONFIGEXPORT)->Count(); itr++)
      {
         const BZBoBase* pBoExp = oBo.GetBoDepList(DRUCKAUSGABEKONFIG_ID::DP_DRUCKAUSGABEKONFIGEXPORT)->GetBO(itr);

         if (!pBoExp->Get(DRUCKAUSGABEKONFIGEXP::SELECTED).Get(false))
            continue;

         if (pBoExp->Get(DRUCKAUSGABEKONFIGEXP::EXPORTKANAL).Left(3) == L"xml")
            ans.druckexportkanaele.Insert(new CString(pBoExp->Get(DRUCKAUSGABEKONFIGEXP::EXPORTKANAL)));
         else
            ans.pdfexportkanaele.Insert(new CString(pBoExp->Get(DRUCKAUSGABEKONFIGEXP::EXPORTKANAL)));
      }
   }
}

//------------------------------------------------------------------------------
bool MgrBsa::IsSWPAktiv()
{
   bool bFound = false;
   return csIni->GetValueBool(L"ISAKTIV", L"EINSTELLUNGEN_SWP", &bFound);
}

//------------------------------------------------------------------------------
bool MgrBsa::IsSWPAktiv(const MModul& oMModul, const bool_null& isNW, const CString& sVerrechnungsfabrikat)
{
   bool bRet      = false;
   bool bIsNW     = false;
   bool bSWPAktiv = IsSWPAktiv();

   if (isNW.IsNull() == true)
   {
      AnsArbBereich oAnsArbBereich;
      GetArbeitsbereich(oMModul, oAnsArbBereich);
      bIsNW = oAnsArbBereich.IsNW();
   }
   else
      bIsNW = isNW.Get(false);
      
   if (bSWPAktiv == true)
   {
      if (bIsNW == false)
      {
         bool bFound = false;
         bRet = csIni->GetValueBool(L"SWP_GW_AKTIV", L"EINSTELLUNGEN_SWP", oMModul, &bFound);
      }
      else
      {
         if (sVerrechnungsfabrikat.IsEmpty() == false)
            bRet = csIni->ExistsKey(sVerrechnungsfabrikat, L"SWP_NW_AKTIV_VERRFABRIKAT", oMModul, true);
      }
   }

   return bRet;
}

//------------------------------------------------------------------------------
void MgrBsa::SetNextFiskalBonNr( const CString& sFiskalBonNrTyp, long lFiskalBonNr )
{
   if (!csIni->GetValueBool(L"FISKALBON_AKTIV", L"EINSTELLUNGEN_FISKALBON"))
      return;

   if (!ZDTsGlobal::ArbeitsplatzDefined())
      BSTHROW BZException(i18n("Arbeitsplatzinfo nicht definiert!"), BsaId::FISKALBON_ARBEITSPLATZ_NOTFOUND);

   BoFiskalBonNummer oBo;
   oBo.Set(FISKALBONNR::ARBEITSPLATZ,  ZDTsGlobal::GetArbeitsplatz());
   oBo.Set(FISKALBONNR::TYPE,          sFiskalBonNrTyp);

   if (!oBo.ReadIfExists())
      BSTHROW BZException(i18n("Kein Fiskalbondruck von diesem Arbeitzplatz aus mglich!"), 
         BsaId::FISKALBON_DATENSATZ_NOTEXIST);

   // Folgende Situation kann auftreten, wenn z.B. eine andere CROSS-Instanz in der Zwischenzeit eine Rechnung gedruckt hat
   // (druckt der Anwender erneut, so sollte dann die neue Nummer dem Anwender angezeigt und diese verwendet werden)
   if (oBo.Get(FISKALBONNR::LETZTEFISKALBONNR).Get(0) + 1 != lFiskalBonNr)
      BSTHROW BZException(i18n("Die Fiskalbonnummer entspricht nicht der nchsten Nummer. Bitte fhren Sie die Aktion erneut aus!"), BsaId::FISKALBON_NUMMER_FALSCH);

   // neue Nummer nun speichern
   oBo.Set(FISKALBONNR::LETZTEFISKALBONNR, lFiskalBonNr);
   oBo.Persist();
}

//------------------------------------------------------------------------------
long MgrBsa::GetUpdateNextBehoerdennummer(const int_null& mandantnr)
{
   BZBoBase oBo(new BEHOERDENNR);
   oBo.Set(BEHOERDENNR::MANDANTNR, mandantnr);
   oBo.Read();

   // neue Nummer nun speichern
   oBo.Set(BEHOERDENNR::LETZTEBEHOERDENNR, oBo.Get(BEHOERDENNR::LETZTEBEHOERDENNR).Get(0) + 1);
   oBo.Persist();

   return oBo.Get(BEHOERDENNR::LETZTEBEHOERDENNR).Get();
}

//------------------------------------------------------------------------------
bool MgrBsa::AutomatischeBehoerdennrVergabeAktiv(const int_null& mandantnr)
{
   if(!csIni->GetValueBool(L"AKTIV_BEHOERDENNR", L"EINSTELLUNGEN_NGS"))
      return false;

   BZBoBase oBo(new BEHOERDENNR);
   oBo.Set(BEHOERDENNR::MANDANTNR, mandantnr);
   if (!oBo.ReadIfExists())
      return false;

   return oBo.Get(BEHOERDENNR::AKTIV).Get();
}

//------------------------------------------------------------------------------
void MgrBsa::GetBehoerdenNr(const int_null& mandantnr, AnsBehoerdenNr& ans)
{
   BZBoBase oBo(new BEHOERDENNR);
   oBo.Set(BEHOERDENNR::MANDANTNR, mandantnr);


   //berschrifte nehmen
   if (oBo.ReadIfExists())
   {
      ans.mandantnr         = oBo.Get(BEHOERDENNR::MANDANTNR);
      ans.letztebehoerdennr = oBo.Get(BEHOERDENNR::LETZTEBEHOERDENNR);
      ans.aktiv             = oBo.Get(BEHOERDENNR::AKTIV);
      ans.ueberschrift1     = oBo.GetString(BEHOERDENNR::UEBERSCHRIFT1);
      ans.ueberschrift2     = oBo.GetString(BEHOERDENNR::UEBERSCHRIFT2);
   }
}

//------------------------------------------------------------------------------
void MgrBsa::GetImageSize( const CString& path, int_null& width, int_null& height )
{
   if ( path.IsEmpty() )
      return;

   int intoX, intoY;
   try
   {
      GetImagesize( path, intoX, intoY );
   }
   catch ( const BSException& ex )
   {
      BSLog( BS_WARN, ex.Message() );
      return;
   }

   width  = intoX;
   height = intoY;
}
